@@ -11,10 +11,15 @@
 
 using namespace QtJson;
 
+Dailymotion* dailymotionInstance = 0;
+
+const QByteArray CLIENT_ID("808ed79fa59e10f3952e");
+const QByteArray CLIENT_SECRET("5753a7aa0fb0772b628d160aabe3da6059f97375");
+const QByteArray REDIRECT_URI("https://sites.google.com/site/marxodian/home/cutetube");
+
 Dailymotion::Dailymotion(QObject *parent) :
-    QObject(parent), m_nam(0),
-    m_clientId("808ed79fa59e10f3952e"),
-    m_clientSecret("5753a7aa0fb0772b628d160aabe3da6059f97375"),
+    QObject(parent),
+    m_nam(0),
     m_safeSearch(false),
     m_actionsProcessed(0),
     m_playlistCache(new QList< QSharedPointer<PlaylistItem> >),
@@ -23,8 +28,13 @@ Dailymotion::Dailymotion(QObject *parent) :
     m_playlistCacheLoaded(false),
     m_subscriptionCacheLoaded(false),
     m_groupCacheLoaded(false),
-    m_busy(false)
+    m_busy(false),
+    m_cancelled(false)
 {
+    if (!dailymotionInstance) {
+        dailymotionInstance = this;
+    }
+
     m_queryOrders[Queries::Relevance] = "relevance";
     m_queryOrders[Queries::Date] = "recent";
     m_queryOrders[Queries::Views] = "visited";
@@ -35,14 +45,19 @@ Dailymotion::~Dailymotion() {
     this->clearCache();
 }
 
+Dailymotion* Dailymotion::instance() {
+    return dailymotionInstance;
+}
+
 void Dailymotion::setBusy(bool isBusy, const QString &message, int numberOfOperations) {
     if (isBusy != this->busy()) {
         m_busy = isBusy;
 
         if (isBusy) {
+            this->setCancelled(false);
             emit busy(message, numberOfOperations);
         }
-        else {
+        else if (!this->cancelled()) {
             emit busyProgressChanged(numberOfOperations);
         }
 
@@ -50,22 +65,34 @@ void Dailymotion::setBusy(bool isBusy, const QString &message, int numberOfOpera
     }
 }
 
-QNetworkReply* Dailymotion::createReply(QString feed, int offset) {
-    QUrl url(feed);
+void Dailymotion::cancelCurrentOperation() {
+    this->setCancelled(true);
+    this->setBusy(false);
+    this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, 0);
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, 0);
+    this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, 0);
+#ifdef QML_USER_INTERFACE
+    this->disconnect(this, SIGNAL(gotVideo(VideoItem*)), 0, 0);
+#else
+    this->disconnect(this, SIGNAL(gotVideo(QSharedPointer<VideoItem>)), 0, 0);
+#endif
+    emit currentOperationCancelled();
+}
 
+QNetworkReply* Dailymotion::createReply(QUrl feed, int offset) {
     if (userSignedIn()) {
-        url.addQueryItem("access_token", this->accessToken());
+        feed.addQueryItem("access_token", this->accessToken());
     }
 
     if (offset) {
-        url.addQueryItem("page", QString::number(offset));
+        feed.addQueryItem("page", QString::number(offset));
     }
 
-    url.addQueryItem("family_filter", this->safeSearch() ? "true" : "false");
+    feed.addQueryItem("family_filter", this->safeSearch() ? "true" : "false");
 
-    qDebug() << url;
+    qDebug() << feed;
 
-    QNetworkRequest request(url);
+    QNetworkRequest request(feed);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
 
     return this->networkAccessManager()->get(request);
@@ -74,26 +101,34 @@ QNetworkReply* Dailymotion::createReply(QString feed, int offset) {
 QNetworkReply* Dailymotion::createSearchReply(int queryType, const QString &query, int offset, int order, const QString &language) {
     QUrl url;
 
-    if (queryType == Queries::Videos) {
+    switch (queryType) {
+    case Queries::Videos:
         url.setUrl(DAILYMOTION_VIDEOS_BASE_URL);
         url.addQueryItem("fields", DAILYMOTION_VIDEO_FIELDS);
         url.addQueryItem("sort", m_queryOrders.value(order, "relevance"));
 
-        if (language != "all") {
+        if ((!language.isEmpty()) && (language != "all")) {
             url.addQueryItem("localization", language);
         }
-    }
-    else if (queryType == Queries::Playlists) {
+        break;
+    case Queries::Playlists:
         url.setUrl(DAILYMOTION_PLAYLISTS_BASE_URL);
         url.addQueryItem("fields", DAILYMOTION_PLAYLIST_FIELDS);
-    }
-    else if (queryType == Queries::Users) {
+        url.addQueryItem("sort", "relevance");
+        break;
+    case Queries::Users:
         url.setUrl(DAILYMOTION_USERS_BASE_URL);
         url.addQueryItem("fields", DAILYMOTION_USER_FIELDS);
-    }
-    else if (queryType == Queries::Groups) {
+        url.addQueryItem("sort", "relevance");
+        break;
+    case Queries::Groups:
         url.setUrl(DAILYMOTION_GROUPS_BASE_URL);
         url.addQueryItem("fields", DAILYMOTION_GROUP_FIELDS);
+        url.addQueryItem("sort", "relevance");
+        break;
+    default:
+        qWarning() << "Dailymotion::createSearchReply(): No/invalid query type.";
+        break;
     }
 
     url.addQueryItem("family_filter", this->safeSearch() ? "true" : "false");
@@ -114,9 +149,9 @@ QNetworkReply* Dailymotion::createSearchReply(int queryType, const QString &quer
 
 QUrl Dailymotion::authUrl() const {
     QUrl url("https://api.dailymotion.com/oauth/authorize");
-    url.addQueryItem("client_id", m_clientId);
+    url.addQueryItem("client_id", CLIENT_ID);
     url.addQueryItem("response_type", "code");
-    url.addQueryItem("redirect_uri", "https://sites.google.com/site/marxodian/home/cutetube");
+    url.addQueryItem("redirect_uri", REDIRECT_URI);
     url.addQueryItem("display", "mobile");
     url.addQueryItem("scope", "read+userinfo+manage_videos+manage_comments+manage_playlists+manage_subscriptions+manage_favorites+manage_groups");
 
@@ -156,15 +191,17 @@ void Dailymotion::signIn(const QString &displayName, const QUrl &response) {
         QUrl url("https://api.dailymotion.com/oauth/token");
 
         QByteArray body("code=" + code.toUtf8() +
-                        "&client_id=" + m_clientId.toUtf8() +
-                        "&client_secret=" + m_clientSecret.toUtf8() +
-                        "&redirect_uri=https://sites.google.com/site/marxodian/home/cutetube&grant_type=authorization_code");
+                        "&client_id=" + CLIENT_ID +
+                        "&client_secret=" + CLIENT_SECRET +
+                        "&redirect_uri=" + REDIRECT_URI +
+                        "&grant_type=authorization_code");
 
         QNetworkRequest request(url);
         request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
         request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
         QNetworkReply *reply = this->networkAccessManager()->post(request, body);
         this->connect(reply, SIGNAL(finished()), this, SLOT(checkIfSignedIn()));
+        this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
     }
 }
 
@@ -175,21 +212,23 @@ void Dailymotion::signIn(const QString &displayName, const QString &user, const
 
     QByteArray body("username=" + user.toUtf8() +
                     "&password=" + pass.toUtf8() +
-                    "&client_id=" + m_clientId.toUtf8() +
-                    "&client_secret=" + m_clientSecret.toUtf8() +
+                    "&client_id=" + CLIENT_ID +
+                    "&client_secret=" + CLIENT_SECRET +
                     "&scope=userinfo+manage_videos+manage_comments+manage_playlists+manage_subscriptions+manage_favorites+manage_groups"
-                    "&redirect_uri=https://sites.google.com/site/marxodian/home/cutetube&grant_type=password");
+                    "&redirect_uri=" + REDIRECT_URI +
+                    "&grant_type=password");
 
     QNetworkRequest request(url);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     QNetworkReply *reply = this->networkAccessManager()->post(request, body);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkIfSignedIn()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void Dailymotion::checkIfSignedIn() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -221,8 +260,8 @@ void Dailymotion::checkIfSignedIn() {
 void Dailymotion::refreshAccessToken() {
     QUrl url("https://api.dailymotion.com/oauth/token");
 
-    QByteArray body("client_id=" + m_clientId.toUtf8() +
-                    "&client_secret=" + m_clientSecret.toUtf8() +
+    QByteArray body("client_id=" + CLIENT_ID +
+                    "&client_secret=" + CLIENT_SECRET +
                     "&refresh_token=" + this->refreshToken().toUtf8() +
                     "&grant_type=refresh_token");
 
@@ -234,7 +273,7 @@ void Dailymotion::refreshAccessToken() {
 }
 
 void Dailymotion::checkTokenRefresh() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -285,11 +324,12 @@ void Dailymotion::signOut() {
     request.setRawHeader("Authorization", "OAuth " + this->accessToken().toUtf8());
     QNetworkReply* reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkIfSignedOut()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void Dailymotion::checkIfSignedOut() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -318,6 +358,7 @@ void Dailymotion::postRequest(const QUrl &url, const QByteArray &data) {
     request.setRawHeader("Authorization", "OAuth " + this->accessToken().toUtf8());
     QNetworkReply* reply = this->networkAccessManager()->post(request, data);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void Dailymotion::deleteRequest(const QUrl &url) {
@@ -327,10 +368,11 @@ void Dailymotion::deleteRequest(const QUrl &url) {
     request.setRawHeader("Authorization", "OAuth " + this->accessToken().toUtf8());
     QNetworkReply* reply = this->networkAccessManager()->deleteResource(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void Dailymotion::postFinished() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -385,7 +427,7 @@ void Dailymotion::getPlaylists(int offset) {
 }
 
 void Dailymotion::addPlaylists() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -417,12 +459,11 @@ void Dailymotion::addPlaylists() {
                 m_playlistCache->append(QSharedPointer<PlaylistItem>(playlist));
             }
 
-            this->setPlaylistsLoaded(true);
-
             if (res.value("has_more").toBool()) {
                 this->getPlaylists(res.value("page").toInt() + 1);
             }
             else {
+                this->setPlaylistsLoaded(true);
                 emit allPlaylistsLoaded();
             }
         }
@@ -487,7 +528,7 @@ void Dailymotion::getGroups(int offset) {
 }
 
 void Dailymotion::addGroups() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -516,16 +557,14 @@ void Dailymotion::addGroups() {
             for (int i = 0; i < entries.size(); i++) {
                 GroupItem *group = new GroupItem;
                 group->loadDailymotionGroup(entries.at(i).toMap(), true);
-                this->connect(this, SIGNAL(groupMembershipChanged(QString,bool)), group, SLOT(onMembershipChanged(QString,bool)));
                 m_groupCache->append(QSharedPointer<GroupItem>(group));
             }
 
-            this->setGroupsLoaded(true);
-
             if (res.value("has_more").toBool()) {
                 this->getGroups(res.value("page").toInt() + 1);
             }
             else {
+                this->setGroupsLoaded(true);
                 emit allGroupsLoaded();
             }
         }
@@ -541,7 +580,6 @@ void Dailymotion::addNewGroupToCache(QSharedPointer<GroupItem> group) {
     m_groupCache->insert(0, group);
     emit groupAddedToCache(0);
     emit groupMembershipChanged(group.data()->id(), true);
-    this->connect(this, SIGNAL(groupMembershipChanged(QString,bool)), group.data(), SLOT(onMembershipChanged(QString,bool)));
 }
 
 QSharedPointer<GroupItem> Dailymotion::removeGroupFromCache(const QString &id) {
@@ -604,7 +642,7 @@ void Dailymotion::getSubscriptions(int offset) {
 }
 
 void Dailymotion::addSubscriptions() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -633,16 +671,14 @@ void Dailymotion::addSubscriptions() {
             for (int i = 0; i < entries.size(); i++) {
                 UserItem *user = new UserItem;
                 user->loadDailymotionUser(entries.at(i).toMap(), true);
-                this->connect(this, SIGNAL(subscriptionChanged(QString,bool)), user, SLOT(onSubscriptionChanged(QString,bool)));
                 m_subscriptionCache->append(QSharedPointer<UserItem>(user));
             }
 
-            this->setSubscriptionsLoaded(true);
-
             if (res.value("has_more").toBool()) {
                 this->getSubscriptions(res.value("page").toInt() + 1);
             }
             else {
+                this->setSubscriptionsLoaded(true);
                 emit allSubscriptionsLoaded();
             }
         }
@@ -655,18 +691,9 @@ void Dailymotion::addSubscriptions() {
 }
 
 void Dailymotion::addNewSubscriptionToCache(QSharedPointer<UserItem> user) {
-    int i = 0;
-    bool found = false;
-
-    while ((!found) && (i < m_subscriptionCache->size())) {
-        found = (m_subscriptionCache->at(i).data()->username() > user.data()->username());
-        i++;
-    }
-
-    m_subscriptionCache->insert(i - 1, user);
-    emit subscriptionAddedToCache(i - 1);
+    m_subscriptionCache->insert(0, user);
+    emit subscriptionAddedToCache(0);
     emit subscriptionChanged(user.data()->id(), true);
-    this->connect(this, SIGNAL(subscriptionChanged(QString,bool)), user.data(), SLOT(onSubscriptionChanged(QString,bool)));
 }
 
 QSharedPointer<UserItem> Dailymotion::removeSubscriptionFromCache(const QString &id) {
@@ -724,17 +751,22 @@ void Dailymotion::getFullVideo(QString id) {
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkFullVideo()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void Dailymotion::checkFullVideo() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
         return;
     }
 
+    if (this->cancelled()) {
+        return;
+    }
+
     int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
     if (statusCode == 200) {
@@ -746,7 +778,6 @@ void Dailymotion::checkFullVideo() {
         if (ok) {
             VideoItem *video = new VideoItem;
             video->loadDailymotionVideo(result);
-            this->connect(this, SIGNAL(favouriteChanged(QString,bool)), video, SLOT(onFavouriteChanged(QString,bool)));
 #ifdef QML_USER_INTERFACE
             QDeclarativeEngine::setObjectOwnership(video, QDeclarativeEngine::JavaScriptOwnership);
             emit gotVideo(video);
@@ -756,12 +787,20 @@ void Dailymotion::checkFullVideo() {
         }
         else {
             emit error(tr("Unable to retrieve video"));
+#ifdef QML_USER_INTERFACE
+            this->disconnect(this, SIGNAL(gotVideo(VideoItem*)), 0, 0);
+#else
             this->disconnect(this, SIGNAL(gotVideo(QSharedPointer<VideoItem>)), 0, 0);
+#endif
         }
     }
     else {
         emit error(tr("Unable to retrieve video"));
+#ifdef QML_USER_INTERFACE
+        this->disconnect(this, SIGNAL(gotVideo(VideoItem*)), 0, 0);
+#else
         this->disconnect(this, SIGNAL(gotVideo(QSharedPointer<VideoItem>)), 0, 0);
+#endif
     }
 
     reply->deleteLater();
@@ -778,7 +817,7 @@ void Dailymotion::getVideoMetadata(const QString &id) {
 }
 
 void Dailymotion::checkVideoMetadata() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -800,7 +839,7 @@ void Dailymotion::checkVideoMetadata() {
         if (!ok) {
             emit error(tr("Error parsing server response"));
         }
-        else {
+        else if (result.value("error").isNull()) {
             QVariantMap metadata;
             metadata.insert("title", result.value("title").toString());
             metadata.insert("description", result.value("description").toString());
@@ -808,9 +847,12 @@ void Dailymotion::checkVideoMetadata() {
             metadata.insert("category", result.value("channel").toString());
             metadata.insert("commentsPermission", result.value("allow_comments").toString());
             metadata.insert("isPrivate", result.value("private").toBool());
-
             emit gotVideoMetadata(metadata);
         }
+        else {
+            QString errorString = result.value("error").toMap().value("message").toString();
+            emit error(errorString.isEmpty() ? tr("Unable to retrieve video metadata") : errorString);
+        }
 
         this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(getVideoMetadata()));
         this->disconnect(this, SIGNAL(postFailed(QString)), this, SIGNAL(error(QString)));
@@ -830,7 +872,7 @@ void Dailymotion::getCurrentUserProfile() {
 }
 
 void Dailymotion::checkCurrentUserProfile() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -885,7 +927,7 @@ void Dailymotion::getUserProfile(const QString &id) {
 }
 
 void Dailymotion::checkUserProfile() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -903,7 +945,6 @@ void Dailymotion::checkUserProfile() {
         if (ok) {
             UserItem *user = new UserItem;
             user->loadDailymotionUser(result);
-            this->connect(this, SIGNAL(subscriptionChanged(QString,bool)), user, SLOT(onSubscriptionChanged(QString,bool)));
 
             if (this->subscriptionsLoaded()) {
                 this->subscribedToChannel(user->id());
@@ -985,22 +1026,24 @@ void Dailymotion::addToFavourites() {
 void Dailymotion::onAddedToFavourites(const QVariantMap &response) {
     Q_UNUSED(response)
 
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit favouriteChanged(m_actionIdList.takeFirst(), true);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->addToFavourites();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) added to favourites"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onAddedToFavourites(QVariantMap)));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(addToFavourites()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->addToFavourites();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) added to favourites"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onAddedToFavourites(QVariantMap)));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(addToFavourites()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1027,22 +1070,24 @@ void Dailymotion::deleteFromFavourites() {
 }
 
 void Dailymotion::onDeletedFromFavourites() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit favouriteChanged(m_actionIdList.takeFirst(), false);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromFavourites();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from favourites"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onDeletedFromFavourites()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromFavourites()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromFavourites();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from favourites"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onDeletedFromFavourites()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromFavourites()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1069,22 +1114,24 @@ void Dailymotion::deleteFromUploads() {
 }
 
 void Dailymotion::onVideoDeleted() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit deletedFromUploads(m_actionIdList.takeFirst());
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromUploads();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from uploads"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onVideoDeleted()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromUploads()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromUploads();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from uploads"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onVideoDeleted()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromUploads()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1115,25 +1162,27 @@ void Dailymotion::addToPlaylist() {
 void Dailymotion::onAddedToPlaylist(const QVariantMap &response) {
     Q_UNUSED(response)
 
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (this->playlistsLoaded()) {
         this->updatePlaylistVideoCount(m_actionId, 1);
     }
     if (!m_actionIdList.isEmpty()) {
         emit addedToPlaylist(m_actionIdList.takeFirst(), m_actionId);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->addToPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) added to playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onAddedToPlaylist(QVariantMap)));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(addToPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->addToPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) added to playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onAddedToPlaylist(QVariantMap)));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(addToPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1160,32 +1209,34 @@ void Dailymotion::deleteFromPlaylist() {
     }
 }
 
-void Dailymotion::onDeletedFromPlaylist() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
+void Dailymotion::onDeletedFromPlaylist() {    
     if (this->playlistsLoaded()) {
         this->updatePlaylistVideoCount(m_actionId, -1);
     }
     if (!m_actionIdList.isEmpty()) {
         emit deletedFromPlaylist(m_actionIdList.takeFirst(), m_actionId);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onDeletedFromPlaylist()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QVariantMap)), this, SLOT(onDeletedFromPlaylist()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString,QString)), this, SLOT(deleteFromPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
 void Dailymotion::createPlaylist(const QVariantMap &playlist, const QStringList &videoIds) {
     m_metadataAction = playlist;
-    m_actionIdList = videoIds;    
+    m_actionIdList = videoIds;
     this->createPlaylist();
 }
 
@@ -1237,7 +1288,7 @@ void Dailymotion::getPlaylistForCache(const QString &id) {
 }
 
 void Dailymotion::checkCachePlaylist() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1329,7 +1380,7 @@ void Dailymotion::getGroupForCache(const QString &id) {
 }
 
 void Dailymotion::checkCacheGroup() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1425,7 +1476,7 @@ void Dailymotion::getAddedComment(const QString &id) {
 }
 
 void Dailymotion::checkAddedComment() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1467,6 +1518,7 @@ void Dailymotion::onSubscribed() {
         this->getSubscriptionForCache(m_actionId);
     }
     else {
+        emit subscriptionChanged(m_actionId, true);
         emit alert(tr("You have subscribed to this channel"));
     }
 
@@ -1485,7 +1537,7 @@ void Dailymotion::getSubscriptionForCache(const QString &id) {
 }
 
 void Dailymotion::checkCacheSubscription() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1535,6 +1587,7 @@ void Dailymotion::onUnsubscribed() {
         QSharedPointer<UserItem> user = this->removeSubscriptionFromCache(m_actionId);
 
         if (!user.isNull()) {
+            emit subscriptionChanged(m_actionId, false);
             emit alert(tr("You have unsubscribed to '%1'").arg(user.data()->username()));
         }
         else {
@@ -1542,6 +1595,7 @@ void Dailymotion::onUnsubscribed() {
         }
     }
     else {
+        emit subscriptionChanged(m_actionId, false);
         emit alert(tr("You have unsubscribed to this channel"));
     }
 
@@ -1587,7 +1641,7 @@ void Dailymotion::onCommentActionError(const QString &errorString) {
 }
 
 void Dailymotion::getVideosFromIds(QStringList ids) {
-    this->setBusy(true);
+    this->setBusy(true, QString(), 0);
     QUrl url(DAILYMOTION_VIDEOS_BASE_URL);
     url.addQueryItem("ids", ids.join(","));
     url.addQueryItem("fields", DAILYMOTION_VIDEO_FIELDS);
@@ -1602,7 +1656,7 @@ void Dailymotion::getVideosFromIds(QStringList ids) {
 
 void Dailymotion::checkFullVideos() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1622,7 +1676,6 @@ void Dailymotion::checkFullVideos() {
             for (int i = 0; i < entries.size(); i++) {
                 VideoItem *video = new VideoItem;
                 video->loadDailymotionVideo(entries.at(i).toMap());
-                this->connect(this, SIGNAL(favouriteChanged(QString,bool)), video, SLOT(onFavouriteChanged(QString,bool)));
                 videos.append(QSharedPointer<VideoItem>(video));
             }
 
