@@ -5,15 +5,10 @@
 #include <QAbstractListModel>
 #include <QSharedPointer>
 
-class Session;
-
 class DailymotionUserGroupListModel : public QAbstractListModel
 {
     Q_OBJECT
 
-    Q_PROPERTY(Session* session
-               READ session
-               WRITE setSession)
     Q_PROPERTY(int count
                READ rowCount
                NOTIFY countChanged)
@@ -28,15 +23,15 @@ public:
     };
 
 public:
-    explicit DailymotionUserGroupListModel(Session *session = 0, QObject *parent = 0);
+    explicit DailymotionUserGroupListModel(QObject *parent = 0);
     ~DailymotionUserGroupListModel() {}
-    inline Session* session() const { return m_session; }
-    void setSession(Session *session);
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     Q_INVOKABLE QVariant data(int row, const QByteArray &role) const;
     QSharedPointer<GroupItem> get(int row) const;
+#ifdef QML_USER_INTERFACE
     Q_INVOKABLE GroupItem* getFromQML(int row) const;
+#endif
 
 private slots:
     void onGroupAdded(int row);
@@ -44,9 +39,6 @@ private slots:
 
 signals:
     void countChanged(int count);
-
-private:
-    Session *m_session;
 };
 
 #endif // DAILYMOTIONUSERGROUPLISTMODEL_H
