@@ -1,7 +1,8 @@
 #include "database.h"
+#include "settings.h"
 #include "utils.h"
 #include <qplatformdefs.h>
-#include <QFile>
+#include <QFileInfo>
 #include <QDir>
 #include <QSqlQuery>
 #include <QSqlRecord>
@@ -12,39 +13,46 @@
 #include <QDebug>
 #include <QDesktopServices>
 
+Database* databaseInstance = 0;
+
 Database::Database(QObject *parent) :
-    QObject(parent)
+    QObject(parent),
+    m_database(QSqlDatabase::addDatabase("QSQLITE"))
 {
+    if (!databaseInstance) {
+        databaseInstance = this;
+    }
 #if (defined (Q_WS_MAEMO_5)) || (defined (MEEGO_EDITION_HARMATTAN))
     QDir dir;
     dir.mkpath("/home/user/cuteTube/");
-    m_database = QSqlDatabase::addDatabase("QSQLITE");
-    m_database.setDatabaseName("/home/user/cuteTube/cuteTube.sqlite");
-#elif (defined (Q_OS_SYMBIAN))
-    m_database = QSqlDatabase::addDatabase("QSQLITE");
-    m_database.setDatabaseName("cuteTube.sqlite");
+    m_database.setDatabaseName("/home/user/cuteTube/cuteTube.db");
+#elif (defined (SYMBIAN_OS))
+    m_database.setDatabaseName("cuteTube.db");
 #else
-    QDir dir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation).append("/cuteTube/"));
+    QDir dir(QDesktopServices::storageLocation(QDesktopServices::HomeLocation) + "/cuteTube/");
     dir.mkpath(dir.path());
-    m_database = QSqlDatabase::addDatabase("QSQLITE");
-    m_database.setDatabaseName(dir.path() + "/cuteTube.sqlite");
+    m_database.setDatabaseName(dir.path() + "/cuteTube.db");
 #endif
-    initialize();
+    this->initialize();
+}
+
+Database* Database::instance() {
+    return databaseInstance;
 }
 
 void Database::initialize() {
     if (m_database.open()) {
-        m_database.exec("DROP TABLE IF EXISTS videoDownloads");
-        m_database.exec("CREATE TABLE IF NOT EXISTS downloads (id TEXT UNIQUE, videoId TEXT, title TEXT, thumbnailUrl TEXT, downloadPath TEXT, tempFileName TEXT, fileName TEXT, service INTEGER, quality INTEGER, priority INTEGER, saveAsAudio INTEGER, rowNumber INTEGER)");
-        m_database.exec("CREATE TABLE IF NOT EXISTS youtubeAccounts (name TEXT UNIQUE, username TEXT, token TEXT, refresh TEXT, cookies TEXT, active INTEGER)");
-        m_database.exec("CREATE TABLE IF NOT EXISTS dailymotionAccounts (name TEXT UNIQUE, username TEXT, token TEXT, refresh TEXT, cookies TEXT, active INTEGER)");
-        m_database.exec("CREATE TABLE IF NOT EXISTS vimeoAccounts (name TEXT UNIQUE, username TEXT, token TEXT, secret TEXT, cookies TEXT, active INTEGER)");
+        m_database.exec("DROP TABLE IF EXISTS downloads");
+        m_database.exec("CREATE TABLE IF NOT EXISTS storedDownloads (id TEXT UNIQUE, videoId TEXT, title TEXT, thumbnailUrl TEXT, downloadPath TEXT, tempFileName TEXT, fileName TEXT, service INTEGER, videoFormat INTEGER, priority INTEGER, saveAsAudio INTEGER, size TEXT)");
+        m_database.exec("CREATE TABLE IF NOT EXISTS youtubeAccounts (name TEXT UNIQUE, username TEXT, token TEXT, refresh TEXT, active INTEGER)");
+        m_database.exec("CREATE TABLE IF NOT EXISTS dailymotionAccounts (name TEXT UNIQUE, username TEXT, token TEXT, refresh TEXT, active INTEGER)");
+        m_database.exec("CREATE TABLE IF NOT EXISTS vimeoAccounts (name TEXT UNIQUE, username TEXT, token TEXT, secret TEXT, active INTEGER)");
         m_database.exec("CREATE TABLE IF NOT EXISTS facebookAccount (name TEXT UNIQUE, token TEXT)");
         m_database.exec("CREATE TABLE IF NOT EXISTS twitterAccount (name TEXT UNIQUE, token TEXT, secret TEXT)");
         m_database.close();
     }
     else {
-        qDebug() << m_database.lastError().text();
+        qWarning() << m_database.lastError().text();
     }
 }
 
@@ -102,38 +110,14 @@ bool Database::setYouTubeAccessToken(const QString &token) {
     }
 }
 
-bool Database::setYouTubeCookies(const QByteArray &cookieString) {
-    if ((m_database.isOpen()) || (m_database.open())) {
-        QSqlQuery query;
-        query.prepare("UPDATE youtubeAccounts SET cookies = ? WHERE active = 1");
-        query.addBindValue(cookieString);
-
-        bool success = query.exec();
-
-        if (!success) {
-            emit error(tr("Database error. Unable to store YouTube cookies"));
-        }
-
-        m_database.close();
-
-        return success;
-    }
-    else {
-        emit error(tr("Database error. Unable to store YouTube cookies"));
-
-        return false;
-    }
-}
-
 void Database::restoreYouTubeAccount() {
     if ((m_database.isOpen()) || (m_database.open())) {
         QString user;
         QString token;
         QString refresh;
-        QByteArray cookieString;
         QSqlQuery query;
         QSqlRecord record;
-        query.exec("SELECT username, token, refresh, cookies FROM youtubeAccounts WHERE active = 1");
+        query.exec("SELECT username, token, refresh FROM youtubeAccounts WHERE active = 1");
         record = query.record();
 
         if (record.count() > 0) {
@@ -141,7 +125,6 @@ void Database::restoreYouTubeAccount() {
                 user = query.value(0).toString();
                 token = query.value(1).toString();
                 refresh = query.value(2).toString();
-                cookieString = query.value(3).toByteArray();
             }
         }
 
@@ -149,10 +132,6 @@ void Database::restoreYouTubeAccount() {
 
         if (!token.isEmpty()) {
             emit gotYouTubeAccount(user, token, refresh);
-
-            if (!cookieString.isEmpty()) {
-                emit gotYouTubeCookies(cookieString);
-            }
         }
     }
 }
@@ -204,38 +183,14 @@ bool Database::setDailymotionAccessToken(const QString &token, const QString &re
     }
 }
 
-bool Database::setDailymotionCookies(const QByteArray &cookieString) {
-    if ((m_database.isOpen()) || (m_database.open())) {
-        QSqlQuery query;
-        query.prepare("UPDATE dailymotionAccounts SET cookies = ? WHERE active = 1");
-        query.addBindValue(cookieString);
-
-        bool success = query.exec();
-
-        if (!success) {
-            emit error(tr("Database error. Unable to store Dailymotion cookies"));
-        }
-
-        m_database.close();
-
-        return success;
-    }
-    else {
-        emit error(tr("Database error. Unable to store Dailymotion cookies"));
-
-        return false;
-    }
-}
-
 void Database::restoreDailymotionAccount() {
     if ((m_database.isOpen()) || (m_database.open())) {
         QString user;
         QString token;
         QString refresh;
-        QByteArray cookieString;
         QSqlQuery query;
         QSqlRecord record;
-        query.exec("SELECT username, token, refresh, cookies FROM dailymotionAccounts WHERE active = 1");
+        query.exec("SELECT username, token, refresh FROM dailymotionAccounts WHERE active = 1");
         record = query.record();
 
         if (record.count() > 0) {
@@ -243,7 +198,6 @@ void Database::restoreDailymotionAccount() {
                 user = query.value(0).toString();
                 token = query.value(1).toString();
                 refresh = query.value(2).toString();
-                cookieString = query.value(3).toByteArray();
             }
         }
 
@@ -251,10 +205,6 @@ void Database::restoreDailymotionAccount() {
 
         if (!token.isEmpty()) {
             emit gotDailymotionAccount(user, token, refresh);
-
-            if (!cookieString.isEmpty()) {
-                emit gotDailymotionCookies(cookieString);
-            }
         }
     }
 }
@@ -306,38 +256,14 @@ bool Database::setVimeoAccessToken(const QString &token, const QString &secret)
     }
 }
 
-bool Database::setVimeoCookies(const QByteArray &cookieString) {
-    if ((m_database.isOpen()) || (m_database.open())) {
-        QSqlQuery query;
-        query.prepare("UPDATE vimeoAccounts SET cookies = ? WHERE active = 1");
-        query.addBindValue(cookieString);
-
-        bool success = query.exec();
-
-        if (!success) {
-            emit error(tr("Database error. Unable to store Vimeo cookies"));
-        }
-
-        m_database.close();
-
-        return success;
-    }
-    else {
-        emit error(tr("Database error. Unable to store Vimeo cookies"));
-
-        return false;
-    }
-}
-
 void Database::restoreVimeoAccount() {
     if ((m_database.isOpen()) || (m_database.open())) {
         QString user;
         QString token;
         QString secret;
-        QByteArray cookieString;
         QSqlQuery query;
         QSqlRecord record;
-        query.exec("SELECT username, token, secret, cookies FROM vimeoAccounts WHERE active = 1");
+        query.exec("SELECT username, token, secret FROM vimeoAccounts WHERE active = 1");
         record = query.record();
 
         if (record.count() > 0) {
@@ -345,7 +271,6 @@ void Database::restoreVimeoAccount() {
                 user = query.value(0).toString();
                 token = query.value(1).toString();
                 secret = query.value(2).toString();
-                cookieString = query.value(3).toByteArray();
             }
         }
 
@@ -353,10 +278,6 @@ void Database::restoreVimeoAccount() {
 
         if (!token.isEmpty()) {
             emit gotVimeoAccount(user, token, secret);
-
-            if (!cookieString.isEmpty()) {
-                emit gotVimeoCookies(cookieString);
-            }
         }
     }
 }
@@ -519,7 +440,7 @@ void Database::restoreTwitterAccount() {
 bool Database::storeDownload(QSharedPointer<TransferItem> transfer) {
     if ((m_database.isOpen()) || (m_database.open())) {
         QSqlQuery query;
-        query.prepare("INSERT OR IGNORE INTO downloads VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+        query.prepare("INSERT OR IGNORE INTO storedDownloads VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
         query.addBindValue(transfer.data()->id());
         query.addBindValue(transfer.data()->videoId());
         query.addBindValue(transfer.data()->title());
@@ -528,10 +449,10 @@ bool Database::storeDownload(QSharedPointer<TransferItem> transfer) {
         query.addBindValue(transfer.data()->tempFileName());
         query.addBindValue(transfer.data()->fileName());
         query.addBindValue(Services::VideoService(transfer.data()->service()));
-        query.addBindValue(Qualities::VideoQuality(transfer.data()->quality()));
+        query.addBindValue(Videos::VideoFormat(transfer.data()->videoFormat()));
         query.addBindValue(Transfers::TransferPriority(transfer.data()->priority()));
         query.addBindValue(transfer.data()->saveAsAudio());
-        query.addBindValue(transfer.data()->row());
+        query.addBindValue(0);
         bool success = query.exec();
         m_database.close();
 
@@ -544,7 +465,7 @@ bool Database::storeDownload(QSharedPointer<TransferItem> transfer) {
 bool Database::removeStoredDownload(const QString &id) {
     if ((m_database.isOpen()) || (m_database.open())) {
         QSqlQuery query;
-        query.prepare("DELETE FROM downloads WHERE id = ?");
+        query.prepare("DELETE FROM storedDownloads WHERE id = ?");
         query.addBindValue(id);
         bool success = query.exec();
         m_database.close();
@@ -558,7 +479,7 @@ bool Database::removeStoredDownload(const QString &id) {
 bool Database::updateStoredDownload(const QString &id, const QString &key, const QVariant &value) {
     if ((m_database.isOpen()) || (m_database.open())) {
         QSqlQuery query;
-        query.prepare(QString("UPDATE downloads SET %1 = ? WHERE id = ?").arg(key));
+        query.prepare(QString("UPDATE storedDownloads SET %1 = ? WHERE id = ?").arg(key));
         query.addBindValue(value);
         query.addBindValue(id);
         bool success = query.exec();
@@ -576,7 +497,7 @@ QList< QSharedPointer<TransferItem> > Database::getStoredDownloads() {
     if ((m_database.isOpen()) || (m_database.open())) {
         QSqlQuery query;
         QSqlRecord record;
-        query.exec("SELECT * FROM downloads ORDER BY rowNumber ASC");
+        query.exec("SELECT * FROM storedDownloads");
         record = query.record();
 
         if (record.count() > 0) {
@@ -590,10 +511,17 @@ QList< QSharedPointer<TransferItem> > Database::getStoredDownloads() {
                 transfer.data()->setTempFileName(query.value(5).toString());
                 transfer.data()->setFileName(query.value(6).toString());
                 transfer.data()->setService(static_cast<Services::VideoService>(query.value(7).toInt()));
-                transfer.data()->setQuality(static_cast<Qualities::VideoQuality>(query.value(8).toInt()));
+                transfer.data()->setVideoFormat(static_cast<Videos::VideoFormat>(query.value(8).toInt()));
                 transfer.data()->setPriority(static_cast<Transfers::TransferPriority>(query.value(9).toInt()));
                 transfer.data()->setSaveAsAudio(query.value(10).toBool());
-                transfer.data()->setRow(query.value(11).toUInt());
+                transfer.data()->setSize(query.value(11).toLongLong());
+
+                QFileInfo file(transfer.data()->downloadPath() + transfer.data()->tempFileName());
+
+                if ((file.size() > 0) && (transfer.data()->size() > 0)) {
+                    transfer.data()->setProgress(file.size() * 100 / transfer.data()->size());
+                }
+
                 transfers.append(transfer);
             }
         }
