@@ -5,6 +5,7 @@
 #include <QSettings>
 #include <QStringList>
 #include <QSet>
+#include <qplatformdefs.h>
 
 class QNetworkProxy;
 class QAuthenticator;
@@ -45,6 +46,10 @@ class Settings : public QSettings
                READ enableYouTube360Download
                WRITE setEnableYouTube360Download
                NOTIFY enableYouTube360DownloadChanged)
+    Q_PROPERTY(bool enableDailymotion1080Playback
+               READ enableDailymotion1080Playback
+               WRITE setEnableDailymotion1080Playback
+               NOTIFY enableDailymotion1080PlaybackChanged)
     Q_PROPERTY(bool enableDailymotion720Playback
                READ enableDailymotion720Playback
                WRITE setEnableDailymotion720Playback
@@ -53,6 +58,10 @@ class Settings : public QSettings
                READ enableDailymotion480Playback
                WRITE setEnableDailymotion480Playback
                NOTIFY enableDailymotion480PlaybackChanged)
+    Q_PROPERTY(bool enableDailymotion1080Download
+               READ enableDailymotion1080Download
+               WRITE setEnableDailymotion1080Download
+               NOTIFY enableDailymotion1080DownloadChanged)
     Q_PROPERTY(bool enableDailymotion720Download
                READ enableDailymotion720Download
                WRITE setEnableDailymotion720Download
@@ -141,18 +150,28 @@ class Settings : public QSettings
                READ showSearchLanguage
                WRITE setShowSearchLanguage
                NOTIFY showSearchLanguageChanged)
+#if (defined (SYMBIAN_OS) || (defined (MEEGO_EDITION_HARMATTAN)))
     Q_PROPERTY(QString theme
                READ theme
                WRITE setTheme
                NOTIFY themeChanged)
     Q_PROPERTY(QString activeColor
                READ activeColor
+#ifdef SYMBIAN_OS
+               CONSTANT)
+#else
                WRITE setActiveColor
                NOTIFY activeColorChanged)
+#endif
     Q_PROPERTY(QString activeColorString
                READ activeColorString
+#ifdef SYMBIAN_OS
+               CONSTANT)
+#else
                WRITE setActiveColorString
                NOTIFY activeColorStringChanged)
+#endif
+#endif
     Q_PROPERTY(QString language
                READ language
                WRITE setLanguage
@@ -169,14 +188,18 @@ class Settings : public QSettings
                READ mediaPlayer
                WRITE setMediaPlayer
                NOTIFY mediaPlayerChanged)
+#ifndef SYMBIAN_OS
     Q_PROPERTY(QString mediaPlayerCommand
                READ mediaPlayerCommand
                WRITE setMediaPlayerCommand
                NOTIFY mediaPlayerCommandChanged)
+#ifdef CUTETUBE_PLAYER_ENABLED
     Q_PROPERTY(bool backgroundPlaybackEnabled
                READ backgroundPlaybackEnabled
                WRITE setBackgroundPlaybackEnabled
                NOTIFY backgroundPlaybackEnabledChanged)
+#endif
+#endif
     Q_PROPERTY(bool monitorClipboard
                READ monitorClipboard
                WRITE setMonitorClipboard
@@ -204,6 +227,7 @@ class Settings : public QSettings
                READ networkProxyPassword
                WRITE setNetworkProxyPassword
                NOTIFY networkProxyPasswordChanged)
+#ifdef Q_WS_MAEMO_5
     Q_PROPERTY(bool eventFeedEnabled
                READ eventFeedEnabled
                WRITE setEventFeedEnabled
@@ -211,6 +235,7 @@ class Settings : public QSettings
     Q_PROPERTY(bool eventFeedInstalled
                READ eventFeedInstalled
                CONSTANT)
+#endif
     Q_PROPERTY(bool cuteTubePlayerEnabled
                READ cuteTubePlayerEnabled
                CONSTANT)
@@ -218,28 +243,30 @@ class Settings : public QSettings
 public:
     explicit Settings(QObject *parent = 0);
     ~Settings();
-    inline QSet<int> youtubePlaybackSet() const { return m_youtubePlaybackSet; }
-    inline QSet<int> youtubeDownloadSet() const { return m_youtubeDownloadSet; }
-    inline QSet<QByteArray> dailymotionPlaybackSet() const { return m_dailymotionPlaybackSet; }
-    inline QSet<QByteArray> dailymotionDownloadSet() const { return m_dailymotionDownloadSet; }
-    inline QSet<QByteArray> vimeoPlaybackSet() const { return m_vimeoPlaybackSet; }
-    inline QSet<QByteArray> vimeoDownloadSet() const { return m_vimeoDownloadSet; }
-    inline bool enableYouTube1080Playback() const { return m_youtubePlaybackSet.contains(37); }
-    inline bool enableYouTube720Playback() const { return m_youtubePlaybackSet.contains(22); }
-    inline bool enableYouTube480Playback() const { return m_youtubePlaybackSet.contains(35); }
-    inline bool enableYouTube360Playback() const { return m_youtubePlaybackSet.contains(34); }
-    inline bool enableYouTube1080Download() const { return m_youtubeDownloadSet.contains(37); }
-    inline bool enableYouTube720Download() const { return m_youtubeDownloadSet.contains(22); }
-    inline bool enableYouTube480Download() const { return m_youtubeDownloadSet.contains(35); }
-    inline bool enableYouTube360Download() const { return m_youtubeDownloadSet.contains(34); }
-    inline bool enableDailymotion720Playback() const { return m_dailymotionPlaybackSet.contains("hd720URL"); }
-    inline bool enableDailymotion480Playback() const { return m_dailymotionPlaybackSet.contains("hqURL"); }
-    inline bool enableDailymotion720Download() const { return m_dailymotionDownloadSet.contains("hd720URL"); }
-    inline bool enableDailymotion480Download() const { return m_dailymotionDownloadSet.contains("hqURL"); }
-    inline bool enableVimeo720Playback() const { return m_vimeoPlaybackSet.contains("hd"); }
-    inline bool enableVimeo360Playback() const { return m_vimeoPlaybackSet.contains("sd"); }
-    inline bool enableVimeo720Download() const { return m_vimeoDownloadSet.contains("hd"); }
-    inline bool enableVimeo360Download() const { return m_vimeoDownloadSet.contains("sd"); }
+    inline QSet<int> youtubePlaybackFormats() const { return m_youtubePlaybackFormats; }
+    inline QSet<int> youtubeDownloadFormats() const { return m_youtubeDownloadFormats; }
+    inline QSet<int> dailymotionPlaybackFormats() const { return m_dailymotionPlaybackFormats; }
+    inline QSet<int> dailymotionDownloadFormats() const { return m_dailymotionDownloadFormats; }
+    inline QSet<int> vimeoPlaybackFormats() const { return m_vimeoPlaybackFormats; }
+    inline QSet<int> vimeoDownloadFormats() const { return m_vimeoDownloadFormats; }
+    inline bool enableYouTube1080Playback() const { return m_youtubePlaybackFormats.contains(Videos::SuperHD); }
+    inline bool enableYouTube720Playback() const { return m_youtubePlaybackFormats.contains(Videos::HD); }
+    inline bool enableYouTube480Playback() const { return m_youtubePlaybackFormats.contains(Videos::HQ); }
+    inline bool enableYouTube360Playback() const { return m_youtubePlaybackFormats.contains(Videos::Normal); }
+    inline bool enableYouTube1080Download() const { return m_youtubeDownloadFormats.contains(Videos::SuperHD); }
+    inline bool enableYouTube720Download() const { return m_youtubeDownloadFormats.contains(Videos::HD); }
+    inline bool enableYouTube480Download() const { return m_youtubeDownloadFormats.contains(Videos::HQ); }
+    inline bool enableYouTube360Download() const { return m_youtubeDownloadFormats.contains(Videos::Normal); }
+    inline bool enableDailymotion1080Playback() const { return m_dailymotionPlaybackFormats.contains(Videos::SuperHD); }
+    inline bool enableDailymotion720Playback() const { return m_dailymotionPlaybackFormats.contains(Videos::HD); }
+    inline bool enableDailymotion480Playback() const { return m_dailymotionPlaybackFormats.contains(Videos::HQ); }
+    inline bool enableDailymotion1080Download() const { return m_dailymotionDownloadFormats.contains(Videos::SuperHD); }
+    inline bool enableDailymotion720Download() const { return m_dailymotionDownloadFormats.contains(Videos::HD); }
+    inline bool enableDailymotion480Download() const { return m_dailymotionDownloadFormats.contains(Videos::HQ); }
+    inline bool enableVimeo720Playback() const { return m_vimeoPlaybackFormats.contains(Videos::HD); }
+    inline bool enableVimeo360Playback() const { return m_vimeoPlaybackFormats.contains(Videos::Normal); }
+    inline bool enableVimeo720Download() const { return m_vimeoDownloadFormats.contains(Videos::HD); }
+    inline bool enableVimeo360Download() const { return m_vimeoDownloadFormats.contains(Videos::Normal); }
     inline bool startTransfersAutomatically() const { return m_autoTransfers; }
     inline int maximumConcurrentTransfers() const { return m_maxConcurrentTransfers; }
     inline QString downloadPath() const { return m_path; }
@@ -256,14 +283,20 @@ public:
     inline bool showSearchType() const { return m_showSearchType; }
     inline QString languageFilter() const { return m_languageFilter; }
     inline bool showSearchLanguage() const { return m_showSearchLanguage; }
+#if (defined (SYMBIAN_OS) || (defined (MEEGO_EDITION_HARMATTAN)))
     inline QString theme() const { return m_theme; }
     inline QString activeColor() const { return m_activeColor; }
     inline QString activeColorString() const { return m_activeColorString; }
+#endif
     inline QString language() const { return m_language; }
     inline QString uploadAttribute() const { return m_attribute; }
     inline QString mediaPlayer() const { return m_mediaplayer; }
+#ifndef SYMBIAN_OS
     inline QString mediaPlayerCommand() const { return m_command; }
+#ifdef CUTETUBE_PLAYER_ENABLED
     inline bool backgroundPlaybackEnabled() const { return m_backgroundPlaybackEnabled; }
+#endif
+#endif
     inline ScreenOrientation::Orientation screenOrientation() const { return m_orientation; }
     inline bool monitorClipboard() const { return m_monitorClipboard; }
     inline NetworkProxyType::ProxyType networkProxyType() const { return m_proxyType; }
@@ -272,10 +305,14 @@ public:
     inline quint16 networkProxyPort() const { return m_proxyPort; }
     inline QString networkProxyUser() const { return m_proxyUser; }
     inline QString networkProxyPassword() const { return m_proxyPass; }
+#ifdef Q_WS_MAEMO_5
     inline bool eventFeedEnabled() const { return m_eventFeedEnabled; }
     bool eventFeedInstalled() const;
+#endif
     inline bool cuteTubePlayerEnabled() const { return m_cuteTubePlayerEnabled; }
 
+    static Settings* instance();
+
 public slots:
     void saveSettings();
     void restoreSettings();
@@ -287,8 +324,10 @@ public slots:
     void setEnableYouTube720Download(bool enable);
     void setEnableYouTube480Download(bool enable);
     void setEnableYouTube360Download(bool enable);
+    void setEnableDailymotion1080Playback(bool enable);
     void setEnableDailymotion720Playback(bool enable);
     void setEnableDailymotion480Playback(bool enable);
+    void setEnableDailymotion1080Download(bool enable);
     void setEnableDailymotion720Download(bool enable);
     void setEnableDailymotion480Download(bool enable);
     void setEnableVimeo720Playback(bool enable);
@@ -311,14 +350,22 @@ public slots:
     void setShowSearchType(bool show);
     void setLanguageFilter(const QString &language);
     void setShowSearchLanguage(bool show);
+#if (defined (SYMBIAN_OS) || (defined (MEEGO_EDITION_HARMATTAN)))
     void setTheme(const QString &theme);
+#ifdef MEEGO_EDITION_HARMATTAN
     void setActiveColor(const QString &color);
     void setActiveColorString(const QString &colorString);
+#endif
+#endif
     void setLanguage(const QString &lang);
     void setUploadAttribute(const QString &attribute);
     void setMediaPlayer(const QString &player);
+#ifndef SYMBIAN_OS
     void setMediaPlayerCommand(const QString &command);
+#ifdef CUTETUBE_PLAYER_ENABLED
     void setBackgroundPlaybackEnabled(bool enabled);
+#endif
+#endif
     void setScreenOrientation(ScreenOrientation::Orientation orientation);
     void setMonitorClipboard(bool monitor);
     void setNetworkProxyType(NetworkProxyType::ProxyType proxyType);
@@ -327,18 +374,19 @@ public slots:
     void setNetworkProxyUser(const QString &user);
     void setNetworkProxyPassword(const QString &password);
     void setNetworkProxy();
+#ifdef Q_WS_MAEMO_5
     void setEventFeedEnabled(bool enabled);
+#endif
 
 private:
-    void setYouTubePlaybackQualitySet(const QVariantList &qualityList);
-    void setYouTubeDownloadQualitySet(const QVariantList &qualityList);
-    void setDailymotionPlaybackQualitySet(const QVariantList &qualityList);
-    void setDailymotionDownloadQualitySet(const QVariantList &qualityList);
-    void setVimeoPlaybackQualitySet(const QVariantList &qualityList);
-    void setVimeoDownloadQualitySet(const QVariantList &qualityList);
+    void setYouTubePlaybackFormats(QVariantList formats);
+    void setYouTubeDownloadFormats(QVariantList formats);
+    void setDailymotionPlaybackFormats(QVariantList formats);
+    void setDailymotionDownloadFormats(QVariantList formats);
+    void setVimeoPlaybackFormats(QVariantList formats);
+    void setVimeoDownloadFormats(QVariantList formats);
+    void setCuteTubePlayerEnabled(bool enabled);
     QVariantList toVariantList(const QSet<int> &set);
-    QVariantList toVariantList(const QSet<QByteArray> &set);
-    void setCuteTubePlayerEnabled();
 
 private slots:
     void onNetworkProxyAuthenticationRequested(const QNetworkProxy &proxy, QAuthenticator *authenticator);
@@ -352,20 +400,22 @@ signals:
     void enableYouTube720DownloadChanged();
     void enableYouTube480DownloadChanged();
     void enableYouTube360DownloadChanged();
+    void enableDailymotion1080PlaybackChanged();
     void enableDailymotion720PlaybackChanged();
     void enableDailymotion480PlaybackChanged();
+    void enableDailymotion1080DownloadChanged();
     void enableDailymotion720DownloadChanged();
     void enableDailymotion480DownloadChanged();
     void enableVimeo720PlaybackChanged();
     void enableVimeo360PlaybackChanged();
     void enableVimeo720DownloadChanged();
     void enableVimeo360DownloadChanged();
-    void youtubePlaybackQualityChanged(const QSet<int> &qualitySet);
-    void youtubeDownloadQualityChanged(const QSet<int> &qualitySet);
-    void dailymotionPlaybackQualityChanged(const QSet<QByteArray> &qualitySet);
-    void dailymotionDownloadQualityChanged(const QSet<QByteArray> &qualitySet);
-    void vimeoPlaybackQualityChanged(const QSet<QByteArray> &qualitySet);
-    void vimeoDownloadQualityChanged(const QSet<QByteArray> &qualitySet);
+    void youtubePlaybackFormatsChanged(const QSet<int> &formats);
+    void youtubeDownloadFormatsChanged(const QSet<int> &formats);
+    void dailymotionPlaybackFormatsChanged(const QSet<int> &formats);
+    void dailymotionDownloadFormatsChanged(const QSet<int> &formats);
+    void vimeoPlaybackFormatsChanged(const QSet<int> &formats);
+    void vimeoDownloadFormatsChanged(const QSet<int> &formats);
     void startTransfersAutomaticallyChanged(bool automatic);
     void maximumConcurrentTransfersChanged(int oldMaximum, int newMaximum);
     void downloadPathChanged(const QString &path);
@@ -382,15 +432,23 @@ signals:
     void showSearchTypeChanged(bool show);
     void languageFilterChanged(const QString &language);
     void showSearchLanguageChanged(bool show);
-    void themeChanged(const QString &theme);
+#if (defined (SYMBIAN_OS) || (defined (MEEGO_EDITION_HARMATTAN)))
+    void themeChanged(const QString &themeString);
+#ifdef MEEGO_EDITION_HARMATTAN
     void activeColorChanged(const QString &color);
     void activeColorStringChanged(const QString &colorString);
+#endif
+#endif
     void languageChanged(const QString &language);
     void alert(const QString &message);
     void uploadAttributeChanged(const QString &attribute);
     void mediaPlayerChanged(const QString &player);
+#ifndef SYMBIAN_OS
     void mediaPlayerCommandChanged(const QString &command);
+#ifdef CUTETUBE_PLAYER_ENABLED
     void backgroundPlaybackEnabledChanged(bool enabled);
+#endif
+#endif
     void screenOrientationChanged(ScreenOrientation::Orientation orientation);
     void monitorClipboardChanged(bool monitor);
     void networkProxyTypeChanged(NetworkProxyType::ProxyType proxyType);
@@ -402,12 +460,12 @@ signals:
     void eventFeedEnabledChanged(bool enabled);
 
 private:
-    QSet<int> m_youtubePlaybackSet;
-    QSet<int> m_youtubeDownloadSet;
-    QSet<QByteArray> m_dailymotionPlaybackSet;
-    QSet<QByteArray> m_dailymotionDownloadSet;
-    QSet<QByteArray> m_vimeoPlaybackSet;
-    QSet<QByteArray> m_vimeoDownloadSet;
+    QSet<int> m_youtubePlaybackFormats;
+    QSet<int> m_youtubeDownloadFormats;
+    QSet<int> m_dailymotionPlaybackFormats;
+    QSet<int> m_dailymotionDownloadFormats;
+    QSet<int> m_vimeoPlaybackFormats;
+    QSet<int> m_vimeoDownloadFormats;
     bool m_autoTransfers;
     int m_maxConcurrentTransfers;
     QString m_path;
@@ -424,14 +482,20 @@ private:
     bool m_showSearchType;
     QString m_languageFilter;
     bool m_showSearchLanguage;
+#if (defined (SYMBIAN_OS) || (defined (MEEGO_EDITION_HARMATTAN)))
     QString m_theme;
     QString m_activeColor;
     QString m_activeColorString;
+#endif
     QString m_language;
     QString m_attribute;
     QString m_mediaplayer;
+#ifndef SYMBIAN_OS
     QString m_command;
+#ifdef CUTETUBE_PLAYER_ENABLED
     bool m_backgroundPlaybackEnabled;
+#endif
+#endif
     ScreenOrientation::Orientation m_orientation;
     bool m_monitorClipboard;
     NetworkProxyType::ProxyType m_proxyType;
@@ -440,8 +504,10 @@ private:
     quint16 m_proxyPort;
     QString m_proxyUser;
     QString m_proxyPass;
+#ifdef Q_WS_MAEMO_5
     bool m_eventFeedEnabled;
     bool m_eventFeedInstalled;
+#endif
     bool m_cuteTubePlayerEnabled;
 };
 
