@@ -9,31 +9,46 @@
 #include <QClipboard>
 #include <QDateTime>
 #include <QDesktopServices>
-#include <qplatformdefs.h>
 #ifdef MEEGO_EDITION_HARMATTAN
 #include <MDataUri>
 #include <maemo-meegotouch-interfaces/shareuiinterface.h>
+#else
+#include "settings.h"
 #endif
 
 using namespace QtJson;
 using namespace QtOAuth;
 
+Sharing* sharingInstance = 0;
+
+const QRegExp YOUTUBE_SHARING_RE("(http(s|)://(www.|m.|)youtube.com/(v/|.+)(v=|)|http://youtu.be/)");
+const QRegExp DAILYMOTION_SHARING_RE("(http(s|)://(www.|)dailymotion.com/video/|http://dai.ly/)\\w{6}");
+const QRegExp VIMEO_SHARING_RE("http(s|)://vimeo.com/\\d+");
+
+const QString FACEBOOK_CLIENT_ID("385476614847489");
+const QString FACEBOOK_CLIENT_SECRET("711d38a113d298af9348386957c524e5");
+const QString TWITTER_CLIENT_ID("YmfKe14OCdVxn9FCL1S77A");
+const QString TWITTER_CLIENT_SECRET("evSW7txyhlvvPXkezKvJgaZ2lC97iXZc4ZLXLZxfsfc");
+const QString REDIRECT_URI("https://sites.google.com/site/marxodian/home/cutetube");
+
 Sharing::Sharing(QObject *parent) :
     QObject(parent),
     m_nam(0),
-    m_facebookClientId("385476614847489"),
-    m_facebookClientSecret("711d38a113d298af9348386957c524e5"),
-    m_twitterClientId("YmfKe14OCdVxn9FCL1S77A"),
-    m_twitterClientSecret("evSW7txyhlvvPXkezKvJgaZ2lC97iXZc4ZLXLZxfsfc"),
-    m_redirectUri("https://sites.google.com/site/marxodian/home/cutetube"),
     m_facebookTimestamp(0),
     m_videoCache(new QList< QSharedPointer<VideoItem> >),
     m_busy(false)
 {
+    if (!sharingInstance) {
+        sharingInstance = this;
+    }
 }
 
 Sharing::~Sharing() {
-    this->clearCache();
+    this->clearCache();    
+}
+
+Sharing* Sharing::instance() {
+    return sharingInstance;
 }
 
 void Sharing::clearCache() {
@@ -55,9 +70,14 @@ void Sharing::setBusy(bool isBusy, const QString &message, int numberOfOperation
     }
 }
 
+void Sharing::cancelCurrentOperation() {
+    this->setBusy(false);
+    emit currentOperationCancelled();
+}
+
 QUrl Sharing::facebookAuthUrl() const {
     QUrl url("https://www.facebook.com/dialog/oauth");
-    url.addQueryItem("client_id", m_facebookClientId);
+    url.addQueryItem("client_id", FACEBOOK_CLIENT_ID);
     url.addQueryItem("redirect_uri", "http://www.facebook.com/connect/login_success.html");
     url.addQueryItem("response_type", "token");
     url.addQueryItem("display", "touch");
@@ -86,9 +106,13 @@ void Sharing::signInToFacebook(const QUrl &response) {
     }
 }
 
+void Sharing::cancelFacebookSignIn() {
+    this->disconnect(SIGNAL(signedInToFacebook(QString)));
+}
+
 QByteArray Sharing::getTwitterOAuthHeader(const QString &method, const QUrl &url, QMap<QString, QString> params) {
-    params.insert("oauth_consumer_key", m_twitterClientId);
-    params.insert("oauth_consumer_secret", m_twitterClientSecret);
+    params.insert("oauth_consumer_key", TWITTER_CLIENT_ID);
+    params.insert("oauth_consumer_secret", TWITTER_CLIENT_SECRET);
 
     if (this->userSignedInToTwitter()) {
         params.insert("oauth_token", this->twitterToken());
@@ -101,7 +125,7 @@ QByteArray Sharing::getTwitterOAuthHeader(const QString &method, const QUrl &url
 void Sharing::getTwitterAuthUrl() {
     QUrl url("https://api.twitter.com/oauth/request_token");
     QMap<QString, QString> params;
-    params.insert("oauth_callback", m_redirectUri);
+    params.insert("oauth_callback", REDIRECT_URI);
     QNetworkRequest request(url);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     request.setRawHeader("Authorization", this->getTwitterOAuthHeader("GET", url, params));
@@ -110,7 +134,7 @@ void Sharing::getTwitterAuthUrl() {
 }
 
 void Sharing::checkTwitterAuthUrl() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -158,7 +182,7 @@ void Sharing::signInToTwitter(const QUrl &response) {
 
 void Sharing::checkIfSignedInToTwitter() {
     this->setBusy(false);
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -182,6 +206,10 @@ void Sharing::checkIfSignedInToTwitter() {
     reply->deleteLater();
 }
 
+void Sharing::cancelTwitterSignIn() {
+    this->disconnect(SIGNAL(signedInToTwitter(QString,QString)));
+}
+
 void Sharing::setFacebookAccessToken(const QString &token) {
     m_facebookToken = token;
     emit userSignedInToFacebookChanged();
@@ -205,109 +233,9 @@ void Sharing::signOutOfTwitter() {
     emit info(tr("Access token deleted. Please visit the twitter website to revoke access"));
 }
 
-void Sharing::postToFacebook(const QVariantMap &post) {
-    m_post = post;
-    this->postToFacebook();
-}
-
-void Sharing::postToFacebook() {
-    QByteArray postData = "access_token=" + this->facebookToken().toUtf8()
-            + "&message=" + m_post.value("body").toString().toUtf8()
-            + "&link=" + m_post.value("videoUrl").toString().toUtf8()
-            + "&source=" + m_post.value("embedUrl").toString().toUtf8()
-            + "&picture=" + m_post.value("thumbnailUrl").toString().toUtf8()
-            + "&name=" + m_post.value("title").toString().toUtf8()
-            + "&description=" + m_post.value("description").toString().toUtf8();
-
-    QNetworkRequest request(QUrl("https://graph.facebook.com/me/feed"));
-    request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    QNetworkReply *reply = this->networkAccessManager()->post(request, postData);
-    this->connect(reply, SIGNAL(finished()), this, SLOT(facebookPostFinished()));
-}
-
-void Sharing::facebookPostFinished() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
-
-    if (!reply) {
-        emit error(tr("Network error"));
-        return;
-    }
-
-    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
-    if ((statusCode == 400) && (userSignedInToFacebook())) {
-        this->setFacebookAccessToken(QString());
-        emit signedOutOfFacebook();
-        emit reauthenticateForFacebook();
-        this->connect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(postToFacebook()));
-    }
-    else {
-        if ((statusCode == 200) || (statusCode == 201)) {
-            emit alert(tr("Video shared to facebook"));
-        }
-        else {
-            QString statusText = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
-            emit error(statusText);
-        }
-
-        this->disconnect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(postToFacebook()));
-    }
-
-    reply->deleteLater();
-}
-
-void Sharing::postToTwitter(const QVariantMap &post) {
-    m_post = post;
-    this->postToTwitter();
-}
-
-void Sharing::postToTwitter() {
-    QByteArray body = m_post.value("body").toString().toUtf8().toPercentEncoding();
-    QUrl url("http://api.twitter.com/1/statuses/update.json");
-    QMap<QString, QString> params;
-    params.insert("status", QString(body));
-    QNetworkRequest request(url);
-    request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
-    request.setRawHeader("Authorization", this->getTwitterOAuthHeader("POST", url, params));
-    QNetworkReply *reply = this->networkAccessManager()->post(request, "status=" + body);
-    this->connect(reply, SIGNAL(finished()), this, SLOT(twitterPostFinished()));
-}
-
-void Sharing::twitterPostFinished() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
-
-    if (!reply) {
-        emit error(tr("Network error"));
-        return;
-    }
-
-    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-
-    if ((statusCode == 401) && (this->userSignedInToTwitter())) {
-        this->setTwitterAccount(QString(), QString());
-        emit signedOutOfTwitter();
-        emit reauthenticateForTwitter();
-        connect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(postToTwitter()));
-    }
-    else {
-        if ((statusCode == 200) || (statusCode == 201)) {
-            emit alert(tr("Video shared to twitter"));
-        }
-        else {
-            QString statusText = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
-            emit error(statusText);
-        }
-
-        this->disconnect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(postToTwitter()));
-    }
-
-    reply->deleteLater();
-}
-
-void Sharing::shareViaHarmattanShareUI(VideoItem *video) {
 #ifdef MEEGO_EDITION_HARMATTAN
+void Sharing::shareViaHarmattanShareUI(VideoItem *video) {
+
     MDataUri duri;
     duri.setMimeType("text/x-url");
     duri.setTextData(video->url().toString());
@@ -332,9 +260,114 @@ void Sharing::shareViaHarmattanShareUI(VideoItem *video) {
     else {
         emit error(tr("Unable to share video"));
     }
+}
 #else
-    Q_UNUSED(video)
-#endif
+void Sharing::postToFacebook(const QVariantMap &post) {
+    m_post = post;
+
+    if (this->userSignedInToFacebook()) {
+        this->postToFacebook();
+    }
+    else {
+        emit reauthenticateForFacebook();
+        this->connect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(postToFacebook()));
+    }
+}
+
+void Sharing::postToFacebook() {
+    QByteArray postData = "access_token=" + this->facebookToken().toUtf8()
+            + "&message=" + m_post.value("body").toString().toUtf8()
+            + "&link=" + m_post.value("videoUrl").toString().toUtf8()
+            + "&source=" + m_post.value("embedUrl").toString().toUtf8()
+            + "&picture=" + m_post.value("thumbnailUrl").toString().toUtf8()
+            + "&name=" + m_post.value("title").toString().toUtf8()
+            + "&description=" + m_post.value("description").toString().toUtf8();
+
+    QNetworkRequest request(QUrl("https://graph.facebook.com/me/feed"));
+    request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+    QNetworkReply *reply = this->networkAccessManager()->post(request, postData);
+    this->connect(reply, SIGNAL(finished()), this, SLOT(facebookPostFinished()));
+    this->disconnect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(postToFacebook()));
+}
+
+void Sharing::facebookPostFinished() {
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
+
+    if (!reply) {
+        emit error(tr("Network error"));
+        return;
+    }
+
+    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    if (statusCode == 400) {
+        emit reauthenticateForFacebook();
+        this->connect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(postToFacebook()));
+    }
+    else {
+        if ((statusCode == 200) || (statusCode == 201)) {
+            emit alert(tr("Video shared to facebook"));
+        }
+        else {
+            QString statusText = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
+            emit error(statusText);
+        }
+    }
+
+    reply->deleteLater();
+}
+
+void Sharing::postToTwitter(const QVariantMap &post) {
+    m_post = post;
+
+    if (this->userSignedInToTwitter()) {
+        this->postToTwitter();
+    }
+    else {
+        emit reauthenticateForTwitter();
+        this->connect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(postToTwitter()));
+    }
+}
+
+void Sharing::postToTwitter() {
+    QUrl url("http://api.twitter.com/1.1/statuses/update.json");
+    QMap<QString, QString> params;
+    params.insert("status", m_post.value("body").toString());
+    QNetworkRequest request(url);
+    request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
+    request.setRawHeader("Authorization", this->getTwitterOAuthHeader("POST", url, params));
+    QNetworkReply *reply = this->networkAccessManager()->post(request, "status=" + m_post.value("body").toString().toUtf8().toPercentEncoding());
+    this->connect(reply, SIGNAL(finished()), this, SLOT(twitterPostFinished()));
+    this->disconnect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(postToTwitter()));
+}
+
+void Sharing::twitterPostFinished() {
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
+
+    if (!reply) {
+        emit error(tr("Network error"));
+        return;
+    }
+
+    int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    if (statusCode == 401) {
+        emit reauthenticateForTwitter();
+        this->connect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(postToTwitter()));
+    }
+    else {
+        if ((statusCode == 200) || (statusCode == 201)) {
+            emit alert(tr("Video shared to twitter"));
+        }
+        else {
+            QString statusText = reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
+            emit error(statusText);
+        }        
+    }
+
+    reply->deleteLater();
 }
 
 void Sharing::shareViaEmail(const QString &title, const QUrl &url) {
@@ -342,10 +375,16 @@ void Sharing::shareViaEmail(const QString &title, const QUrl &url) {
 }
 
 void Sharing::copyToClipboard(const QUrl &url) {
+    bool monitor = Settings::instance()->monitorClipboard();
+    Settings::instance()->setMonitorClipboard(false);
     QApplication::clipboard()->setText(url.toString());
     emit alert(tr("Copied to clipboard"));
-}
 
+    if (monitor) {
+        Settings::instance()->setMonitorClipboard(true);
+    }
+}
+#endif
 void Sharing::getVideos(bool forceRefresh) {
     if ((forceRefresh) || (this->videoRefreshIsDue())) {
         if (this->userSignedInToFacebook()) {
@@ -364,7 +403,7 @@ void Sharing::getVideos(bool forceRefresh) {
 }
 
 void Sharing::getVideosFromFacebook() {
-    this->setBusy(true, tr("Retrieving videos from facebook"));
+    this->setBusy(true);
     QUrl url("https://graph.facebook.com/me/feed");
 
     if (this->facebookTimeStamp() > 0) {
@@ -377,11 +416,12 @@ void Sharing::getVideosFromFacebook() {
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkFacebookVideos()));
+    this->disconnect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(getVideosFromFacebook()));
 }
 
 void Sharing::checkFacebookVideos() {
     this->setBusy(false);
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
     if (!reply) {
         emit error("Network error");
         return;
@@ -390,8 +430,6 @@ void Sharing::checkFacebookVideos() {
     int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
     if (statusCode == 400) {
-        this->setFacebookAccessToken(QString());
-        emit signedOutOfFacebook();
         emit reauthenticateForFacebook();
         this->connect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(getVideosFromFacebook()));
     }
@@ -408,18 +446,20 @@ void Sharing::checkFacebookVideos() {
             QVariantList results = result.value("data").toList();
             QStringList youtubeIds;
             QStringList dailymotionIds;
-            QRegExp ytre("(youtube|youtu.be)", Qt::CaseInsensitive);
-            QRegExp dmre("(dailymotion|dai.ly)", Qt::CaseInsensitive);
+            QStringList vimeoIds;
 
             for (int i = 0; i < results.size(); i++) {
                 QString link = results.at(i).toMap().value("link").toString();
 
                 if (!link.isEmpty()) {
-                    if (link.contains(ytre)) {
-                        youtubeIds.append(link.section(QRegExp("(v=|/)"), -1).left(11));
+                    if (link.contains(YOUTUBE_SHARING_RE)) {
+                        youtubeIds.append(link.section(QRegExp("v=|/"), -1).section(QRegExp("&|\\?"), 0, 0));
                     }
-                    else if (link.contains(dmre)) {
-                        dailymotionIds.append(link.section("/", -1).left(6));
+                    else if (link.contains(DAILYMOTION_SHARING_RE)) {
+                        dailymotionIds.append(link.section('/', -1).left(6));
+                    }
+                    else if (link.contains(VIMEO_SHARING_RE)) {
+                        vimeoIds.append(link.section('/', -1));
                     }
                 }
             }
@@ -432,53 +472,46 @@ void Sharing::checkFacebookVideos() {
                 dailymotionIds.removeDuplicates();
                 emit gotDailymotionIds(dailymotionIds);
             }
+            if (!vimeoIds.isEmpty()) {
+                vimeoIds.removeDuplicates();
+                emit gotVimeoIds(vimeoIds);
+            }
         }
 
-        if (userSignedInToTwitter()) {
+        if (this->userSignedInToTwitter()) {
             this->getVideosFromTwitter();
         }
         else {
             this->resetVideoRefreshTime();
-        }
-
-        this->disconnect(this, SIGNAL(signedInToFacebook(QString)), this, SLOT(getVideosFromFacebook()));
+        }        
     }
 
     reply->deleteLater();
 }
 
 void Sharing::getVideosFromTwitter() {
-    this->setBusy(true, tr("Retreiving videos from twitter"));
-    QUrl url("http://api.twitter.com/1/statuses/home_timeline.json");
+    this->setBusy(true);
+    QUrl url("http://api.twitter.com/1.1/statuses/home_timeline.json");
     url.addQueryItem("count", "200");
     url.addQueryItem("include_entities", "true");
     url.addQueryItem("trim_user", "true");
     url.addQueryItem("include_rts", "false");
 
-    if (!lastTweet().isEmpty()) {
+    if (!this->lastTweet().isEmpty()) {
         url.addQueryItem("since_id", this->lastTweet());
     }
 
-    QMap<QString, QString> params;
-    params.insert("count", "200");
-    params.insert("include_entities", "true");
-    params.insert("trim_user", "true");
-    params.insert("include_rts", "false");
-
-    if (!lastTweet().isEmpty()) {
-        params.insert("since_id", lastTweet());
-    }
-
     QNetworkRequest request(url);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    request.setRawHeader("Authorization", this->getTwitterOAuthHeader("GET", QUrl("http://api.twitter.com/1/statuses/home_timeline.json"), params));
+    request.setRawHeader("Authorization", this->getTwitterOAuthHeader("GET", url, QMap<QString, QString>()));
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkTwitterVideos()));
+    this->disconnect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(getVideosFromTwitter()));
 }
 
 void Sharing::checkTwitterVideos() {
     this->setBusy(false);
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -487,9 +520,7 @@ void Sharing::checkTwitterVideos() {
 
     int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
-    if ((statusCode == 401) && (this->userSignedInToTwitter())) {
-        this->setTwitterAccount(QString(), QString());
-        emit signedOutOfTwitter();
+    if (statusCode == 401) {
         emit reauthenticateForTwitter();
         this->connect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(getVideosFromTwitter()));
     }
@@ -504,8 +535,7 @@ void Sharing::checkTwitterVideos() {
         else {
             QStringList youtubeIds;
             QStringList dailymotionIds;
-            QRegExp ytre("(youtube|youtu.be)", Qt::CaseInsensitive);
-            QRegExp dmre("(dailymotion|dai.ly)", Qt::CaseInsensitive);
+            QStringList vimeoIds;
 
             if (!results.isEmpty()) {
                 this->setLastTweet(results.first().toMap().value("id_str").toString());
@@ -519,11 +549,14 @@ void Sharing::checkTwitterVideos() {
                     QString url = urls.takeFirst().toMap().value("expanded_url").toString();
 
                     if (!url.isEmpty()) {
-                        if (url.contains(ytre)) {
-                            youtubeIds.append(url.section(QRegExp("(v=|/)"), -1).left(11));
+                        if (url.contains(YOUTUBE_SHARING_RE)) {
+                            youtubeIds.append(url.section(QRegExp("v=|/"), -1).section(QRegExp("&|\\?"), 0, 0));
                         }
-                        else if (url.contains(dmre)) {
-                            dailymotionIds.append(url.section("/", -1).left(6));
+                        else if (url.contains(DAILYMOTION_SHARING_RE)) {
+                            dailymotionIds.append(url.section('/', -1).left(6));
+                        }
+                        else if (url.contains(VIMEO_SHARING_RE)) {
+                            vimeoIds.append(url.section('/', -1));
                         }
                     }
                 }
@@ -537,9 +570,11 @@ void Sharing::checkTwitterVideos() {
                 dailymotionIds.removeDuplicates();
                 emit gotDailymotionIds(dailymotionIds);
             }
+            if (!vimeoIds.isEmpty()) {
+                vimeoIds.removeDuplicates();
+                emit gotVimeoIds(vimeoIds);
+            }
         }
-
-        this->disconnect(this, SIGNAL(signedInToTwitter(QString,QString)), this, SLOT(getVideosFromTwitter()));
     }
 
     this->resetVideoRefreshTime();
@@ -551,6 +586,4 @@ void Sharing::addVideosToCache(QList< QSharedPointer<VideoItem> > videos) {
         m_videoCache->insert(0, videos.takeFirst());
         emit videoAddedToCache(0);
     }
-
-    emit videosAdded();
 }
