@@ -5,16 +5,14 @@
 #include <QAbstractListModel>
 #include <QSharedPointer>
 
-class Session;
+#ifndef QML_USER_INTERFACE
 class ThumbnailCache;
+#endif
 
 class VimeoUserGroupListModel : public QAbstractListModel
 {
     Q_OBJECT
 
-    Q_PROPERTY(Session* session
-               READ session
-               WRITE setSession)
     Q_PROPERTY(int count
                READ rowCount
                NOTIFY countChanged)
@@ -26,35 +24,38 @@ public:
         ArtistRole,
         DateRole,
         VideoCountRole,
+#ifndef QML_USER_INTERFACE
         ThumbnailRole,
+#endif
         ThumbnailUrlRole,
         DescriptionRole
     };
 
 public:
-    explicit VimeoUserGroupListModel(Session *session = 0, QObject *parent = 0);
-    ~VimeoUserGroupListModel() {}
-    inline Session* session() const { return m_session; }
-    void setSession(Session *session);
-    inline ThumbnailCache* thumbnailCache() const { return m_cache; }
-    void setThumbnailCache(ThumbnailCache *cache);
+    explicit VimeoUserGroupListModel(QObject *parent = 0);
+    ~VimeoUserGroupListModel();
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     Q_INVOKABLE QVariant data(int row, const QByteArray &role) const;
     QSharedPointer<GroupItem> get(int row) const;
+#ifdef QML_USER_INTERFACE
     Q_INVOKABLE GroupItem* getFromQML(int row) const;
+#endif
 
 private slots:
     void onGroupAdded(int row);
     void onGroupRemoved(int row);
+#ifndef QML_USER_INTERFACE
     void onThumbnailReady();
+#endif
 
 signals:
     void countChanged(int count);
 
+#ifndef QML_USER_INTERFACE
 private:
-    Session *m_session;
     ThumbnailCache *m_cache;
+#endif
 };
 
 #endif // VIMEOUSERGROUPLISTMODEL_H
