@@ -5,16 +5,14 @@
 #include <QAbstractListModel>
 #include <QSharedPointer>
 
-class Session;
+#ifndef QML_USER_INTERFACE
 class ThumbnailCache;
+#endif
 
 class YouTubeUserPlaylistListModel : public QAbstractListModel
 {
     Q_OBJECT
 
-    Q_PROPERTY(Session* session
-               READ session
-               WRITE setSession)
     Q_PROPERTY(int count
                READ rowCount
                NOTIFY countChanged)
@@ -27,36 +25,39 @@ public:
         ArtistRole,
         DateRole,
         VideoCountRole,
+#ifndef QML_USER_INTERFACE
         ThumbnailRole,
+#endif
         ThumbnailUrlRole,
         DescriptionRole
     };
 
 public:
-    explicit YouTubeUserPlaylistListModel(Session *session = 0, QObject *parent = 0);
+    explicit YouTubeUserPlaylistListModel(QObject *parent = 0);
     ~YouTubeUserPlaylistListModel();
-    inline Session* session() const { return m_session; }
-    void setSession(Session *session);
-    inline ThumbnailCache* thumbnailCache() const { return m_cache; }
-    void setThumbnailCache(ThumbnailCache *cache);
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
     Q_INVOKABLE QVariant data(int row, const QByteArray &role) const;
     QSharedPointer<PlaylistItem> get(int row) const;
+#ifdef QML_USER_INTERFACE
     Q_INVOKABLE PlaylistItem* getFromQML(int row) const;
+#endif
 
 private slots:
     void onPlaylistAdded(int row);
     void onPlaylistRemoved(int row);
     void onPlaylistUpdated(int row);
+#ifndef QML_USER_INTERFACE
     void onThumbnailReady();
+#endif
 
 signals:
     void countChanged(int count);
 
+#ifndef QML_USER_INTERFACE
 private:
-    Session *m_session;
     ThumbnailCache *m_cache;
+#endif
 };
 
 #endif // YOUTUBEUSERPLAYLISTLISTMODEL_H
