@@ -26,7 +26,7 @@ private slots:
     void onTransferStatusInfoChanged(const QString &info);
     void onTransferSizeChanged(qint64 size);
     void onTransferProgressChanged(int progress);
-    void onTransferSpeedChanged(double speed);
+    void onTransferSpeedChanged(int speed);
     void onFileNameTextChanged(const QString &text);
     void setTransferFileName();
     void onPauseResumeClicked();
