@@ -4,6 +4,7 @@
 #include "listview.h"
 #include "separatorlabel.h"
 #include <QVBoxLayout>
+#include <QUrl>
 
 VimeoCategoriesDialog::VimeoCategoriesDialog(QWidget *parent) :
     AbstractDialog(parent),
@@ -13,7 +14,7 @@ VimeoCategoriesDialog::VimeoCategoriesDialog(QWidget *parent) :
     this->setWindowTitle(tr("Vimeo categories"));
     m_view->setModel(m_model);
     m_view->setModel(m_model);
-    m_view->setItemDelegate(new TextListDelegate(Qt::DisplayRole, Qt::AlignVCenter, m_view));
+    m_view->setItemDelegate(new TextListDelegate(Qt::DisplayRole, Qt::AlignCenter, m_view));
     m_view->setMinimumHeight(m_view->sizeHintForRow(0) * 5);
 
     QVBoxLayout *vbox = new QVBoxLayout(this);
@@ -23,6 +24,6 @@ VimeoCategoriesDialog::VimeoCategoriesDialog(QWidget *parent) :
 }
 
 void VimeoCategoriesDialog::onCategoryChosen(const QModelIndex &index) {
-    emit categoryChosen(Services::Vimeo, m_model->name(index.row()), m_model->value(index.row()).toString());
+    emit categoryChosen(Services::Vimeo, m_model->name(index.row()), m_model->value(index.row()).toUrl());
     this->accept();
 }
