@@ -6,7 +6,6 @@
 #include <QVariantMap>
 
 class VimeoVideoListModel;
-class Session;
 class ListView;
 class QMaemo5EditBar;
 class QPushButton;
@@ -16,8 +15,9 @@ class QModelIndex;
 class VimeoVideoSelectionWindow : public QWidget
 {
     Q_OBJECT
+
 public:
-    explicit VimeoVideoSelectionWindow(Session *session, VimeoVideoListModel *model, const QString &text, const char* slot, QWidget *parent);
+    explicit VimeoVideoSelectionWindow(VimeoVideoListModel *model, const QString &text, const char* slot, QWidget *parent);
     ~VimeoVideoSelectionWindow();
     void positionAtIndex(const QModelIndex &index);
 
@@ -41,7 +41,6 @@ signals:
     void accepted();
 
 private:
-    Session *m_session;
     VimeoVideoListModel *m_model;
     ListView *m_listView;
     QMaemo5EditBar *m_editBar;
