@@ -3,20 +3,21 @@
 
 #include "abstracttab.h"
 
-class Session;
 class YouTubeUserListModel;
 class ListView;
 class QLabel;
 class QGridLayout;
 class QAction;
 class QModelIndex;
+class QUrl;
 
 class YouTubeUserListTab : public AbstractTab
 {
     Q_OBJECT
+
 public:
-    explicit YouTubeUserListTab(Session *session, const QString &userId, QWidget *parent = 0);
-    void setFeed(const QString &feed);
+    explicit YouTubeUserListTab(const QString &userId, QWidget *parent = 0);
+    void setFeed(const QUrl &feed);
     
 private:
     void setLandscapeLayout();
@@ -30,7 +31,6 @@ private slots:
     void showSubscriptionVideos();
 
 private:
-    Session *m_session;
     QString m_userId;
     YouTubeUserListModel *m_model;
     ListView *m_listView;
