@@ -4,9 +4,7 @@
 #include "dailymotion.h"
 #include "vimeo.h"
 #include "database.h"
-#include <qplatformdefs.h>
 #include <QUrl>
-#include <QSettings>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
 #include <QNetworkRequest>
@@ -19,7 +17,8 @@
 #include <QFileInfo>
 #include <QFile>
 #include <QDir>
-#include <QDebug>
+#include <QDateTime>
+#include <QSettings>
 #ifdef MEEGO_EDITION_HARMATTAN
 #include <QNetworkConfigurationManager>
 #include <QNetworkSession>
@@ -49,11 +48,10 @@ void Events::init() {
     m_vimeo->setNetworkAccessManager(m_nam);
 #ifdef MEEGO_EDITION_HARMATTAN
     QNetworkConfigurationManager configManager;
-    QNetworkSession *session = new QNetworkSession(configManager.defaultConfiguration());
+    QNetworkSession *session = new QNetworkSession(configManager.defaultConfiguration(), this);
     session->setSessionProperty("ConnectInBackground", true);
     this->connect(session, SIGNAL(error(QNetworkSession::SessionError)), this, SLOT(syncFailed()));
     this->connect(this, SIGNAL(destroyed()), session, SLOT(close()));
-    this->connect(this, SIGNAL(destroyed()), session, SLOT(deleteLater()));
     session->open();
 #endif
     this->connect(m_database, SIGNAL(gotYouTubeAccount(QString,QString,QString)), m_youtube, SLOT(setAccount(QString,QString,QString)));
@@ -72,8 +70,8 @@ void Events::init() {
 }
 
 bool Events::enableSync() {
-#ifdef MEEGO_EDITION_HARMATTAN
     QSettings settings("cuteTube", "cuteTube");
+#ifdef MEEGO_EDITION_HARMATTAN
     settings.setValue("Events/eventFeedEnabled", true);
     bool success = true;
 #else
@@ -81,14 +79,15 @@ bool Events::enableSync() {
     bool success = cookie > 0;
 
     if (success) {
-        QSettings settings("cuteTube", "cuteTube");
         settings.setValue("Events/eventsAlarmCookie", cookie);
         settings.setValue("Events/eventFeedEnabled", true);
         QDBusInterface iface("com.maemo.eventFeed", "/", "com.maemo.eventFeed");
         iface.call(QDBus::NoBlock, "addRefreshAction", "cutetube", "dbus:session com.maemo.cuteTubeEvents / com.maemo.cuteTubeEvents startSync");
     }
 #endif
-    QTimer::singleShot(1000, QCoreApplication::instance(), SLOT(quit()));
+    if (success) {
+        this->startSync();
+    }
 
     return success;
 }
@@ -96,11 +95,12 @@ bool Events::enableSync() {
 bool Events::disableSync() {
     QSettings settings("cuteTube", "cuteTube");
 #ifdef MEEGO_EDITION_HARMATTAN
-    MEventFeed::instance()->removeItemsBySourceName("cutetube");
-    settings.setValue("Events/youtubeLatest", 0);
-    settings.setValue("Events/dailymotionLatest", 0);
-    settings.setValue("Events/vimeoLatest", 0);
+    settings.remove("Events/youtubeLatest");
+    settings.remove("Events/dailymotionLatest");
+    settings.remove("Events/vimeoLatest");
     settings.setValue("Events/eventFeedEnabled", false);
+    MEventFeed::instance()->removeItemsBySourceName("cutetube");
+    this->clearThumbnails();
     bool success = true;
 #else
     qint64 cookie = settings.value("Events/eventsAlarmCookie", 0).toLongLong();
@@ -110,10 +110,10 @@ bool Events::disableSync() {
         success = delete_alarm(cookie);
 
         if (success) {
-            settings.setValue("Events/eventsAlarmCookie", 0);
-            settings.setValue("Events/youtubeLatest", 0);
-            settings.setValue("Events/dailymotionLatest", 0);
-            settings.setValue("Events/vimeoLatest", 0);
+            settings.remove("Events/eventsAlarmCookie");
+            settings.remove("Events/youtubeLatest");
+            settings.remove("Events/dailymotionLatest");
+            settings.remove("Events/vimeoLatest");
             settings.setValue("Events/eventFeedEnabled", false);
             QDBusInterface iface("com.maemo.eventFeed", "/", "com.maemo.eventFeed");
             iface.call(QDBus::NoBlock, "removeEventsBySourceName", "cutetube");
@@ -131,11 +131,6 @@ void Events::startSync() {
         this->init();
     }
 
-    QSettings settings("cuteTube", "cuteTube");
-    m_youtubeLatest = QDateTime::fromMSecsSinceEpoch(settings.value("Events/youtubeLatest", 0).toLongLong());
-    m_dailymotionLatest = QDateTime::fromMSecsSinceEpoch(settings.value("Events/dailymotionLatest", 0).toLongLong());
-    m_vimeoLatest = QDateTime::fromMSecsSinceEpoch(settings.value("Events/vimeoLatest", 0).toLongLong());
-
     this->getEvents();
 }
 
@@ -144,10 +139,6 @@ void Events::abortSync() {
 }
 
 void Events::syncSuccess() {
-    QSettings settings("cuteTube", "cuteTube");
-    settings.setValue("Events/youtubeLatest", m_youtubeLatest.toMSecsSinceEpoch());
-    settings.setValue("Events/dailymotionLatest", m_dailymotionLatest.toMSecsSinceEpoch());
-    settings.setValue("Events/vimeoLatest", m_vimeoLatest.toMSecsSinceEpoch());
     QCoreApplication::instance()->quit();
 }
 
@@ -168,13 +159,13 @@ void Events::getEvents() {
 }
 
 void Events::getYouTubeEvents() {
-    qDebug() << "Getting YouTube events";
-    QNetworkReply *reply = m_youtube->createReply(QString("https://gdata.youtube.com/feeds/api/users/default/newsubscriptionvideos?v=2.1&max-results=50&fields=openSearch:totalResults,entry[xs:dateTime(published) > xs:dateTime('%1')](id,media:group(media:title,media:thumbnail[@yt:name='default'or@yt:name='hqdefault'],media:description,media:player[@url],media:credit,yt:uploaded,yt:uploaderId,yt:videoid))").arg(m_youtubeLatest.toString(Qt::ISODate)));
+    QDateTime youtubeLatest = QDateTime::fromMSecsSinceEpoch(QSettings("cuteTube", "cuteTube").value("Events/youtubeLatest", 0).toLongLong());
+    QNetworkReply *reply = m_youtube->createReply(QString("https://gdata.youtube.com/feeds/api/users/default/newsubscriptionvideos?v=2.1&max-results=50&fields=openSearch:totalResults,entry[xs:dateTime(published) > xs:dateTime('%1')](id,media:group(media:title,media:thumbnail[@yt:name='default'or@yt:name='hqdefault'],media:description,media:player[@url],media:credit,yt:uploaded,yt:uploaderId,yt:videoid))").arg(youtubeLatest.toString(Qt::ISODate)));
     this->connect(reply, SIGNAL(finished()), this, SLOT(processYouTubeEvents()));
 }
 
 void Events::processYouTubeEvents() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         this->syncFailed();
@@ -205,15 +196,15 @@ void Events::processYouTubeEvents() {
             event.setFooter(mediaGroup.firstChildElement("media:credit").attribute("yt:display"));
 #ifdef MEEGO_EDITION_HARMATTAN
             event.setTimestamp(QDateTime::fromString(mediaGroup.firstChildElement("yt:uploaded").text(), Qt::ISODate).toLocalTime().toString("yyyy-MM-dd hh:mm:ss"));
-            event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube displayVideo %1").arg(base64SerializedVariant(QVariant("http://youtu.be/" + event.id()))));
+            event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube display %1").arg(this->base64SerializedVariant(QVariant("http://youtu.be/" + event.id()))));
 #else
             event.setTimestamp(mediaGroup.firstChildElement("yt:uploaded").text());
-            event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube displayVideo http://youtu.be/" + event.id());
+            event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube display http://youtu.be/" + event.id());
 #endif
             m_events.append(event);
 
             if (i == 0) {
-                m_youtubeLatest = QDateTime::fromString(event.timestamp(), Qt::ISODate);
+                QSettings("cuteTube", "cuteTube").setValue("Events/youtubeLatest", QDateTime::fromString(event.timestamp(), Qt::ISODate).toMSecsSinceEpoch());
             }
         }
 
@@ -233,13 +224,12 @@ void Events::processYouTubeEvents() {
 }
 
 void Events::getDailymotionEvents() {
-    qDebug() << "Getting Dailymotion events";
     QNetworkReply *reply = m_dailymotion->createReply("https://api.dailymotion.com/me/subscriptions?limit=50&fields=created_time,description,id,owner.username,owner.id,thumbnail_medium_url,thumbnail_large_url,title,url");
     this->connect(reply, SIGNAL(finished()), this, SLOT(processDailymotionEvents()));
 }
 
 void Events::processDailymotionEvents() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         this->syncFailed();
@@ -261,13 +251,14 @@ void Events::processDailymotionEvents() {
             return;
         }
 
+        qint64 dailymotionLatest = QSettings("cuteTube", "cuteTube").value("Events/dailymotionLatest", 0).toLongLong();
         QVariantList entries = res.value("list").toList();
 
         for (int i = 0; i < entries.size(); i++) {
             QVariantMap entry = entries.at(i).toMap();
-            QDateTime date = QDateTime::fromMSecsSinceEpoch(entry.value("created_time").toLongLong() * 1000);
+            qint64 date = entry.value("created_time").toLongLong() * 1000;
 
-            if (date > m_dailymotionLatest) {
+            if (date > dailymotionLatest) {
                 Event event;
                 event.setId(entry.value("id").toString());
                 event.setSourceName("cutetube");
@@ -275,21 +266,21 @@ void Events::processDailymotionEvents() {
                 event.setIconPath("/usr/share/icons/hicolor/64x64/apps/cutetube64.png");
                 event.setTitle(entry.value("title").toString());
                 event.setBody(entry.value("description").toString());
-                event.setImagePathOne(entry.value("thumbnail_medium_url").toString());
+                event.setImagePathOne(entry.value("thumbnail_large_url").toString());
                 event.setFooter(entry.value("owner.username").toString());
 #ifdef MEEGO_EDITION_HARMATTAN
-                event.setTimestamp(date.toLocalTime().toString("yyyy-MM-dd hh:mm:ss"));
-                event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube displayVideo %1").arg(base64SerializedVariant(QVariant("http://dai.ly/" + event.id()))));
+                event.setTimestamp(QDateTime::fromMSecsSinceEpoch(date).toLocalTime().toString("yyyy-MM-dd hh:mm:ss"));
+                event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube display %1").arg(this->base64SerializedVariant(QVariant("http://dai.ly/" + event.id()))));
 #else
-                event.setTimestamp(QString::number(date.toMSecsSinceEpoch()));
-                event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube displayVideo http://dai.ly/" + event.id());
+                event.setTimestamp(QString::number(date));
+                event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube display http://dai.ly/" + event.id());
 #endif
                 m_events.append(event);
             }
         }
         
         if (!entries.isEmpty()) {
-            m_dailymotionLatest = QDateTime::fromMSecsSinceEpoch(entries.first().toMap().value("created_time").toLongLong() * 1000);
+            QSettings("cuteTube", "cuteTube").setValue("Events/dailymotionLatest", entries.first().toMap().value("created_time").toLongLong() * 1000);
         }
 
         if (m_vimeo->userSignedIn()) {
@@ -308,13 +299,12 @@ void Events::processDailymotionEvents() {
 }
 
 void Events::getVimeoEvents() {
-    qDebug() << "Getting vimeo events";
     QNetworkReply *reply = m_vimeo->createReply("http://vimeo.com/api/rest/v2?method=vimeo.videos.getSubscriptions&per_page=50&full_response=true");
     this->connect(reply, SIGNAL(finished()), this, SLOT(processVimeoEvents()));
 }
 
 void Events::processVimeoEvents() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         this->syncFailed();
@@ -326,13 +316,14 @@ void Events::processVimeoEvents() {
     QDomElement responseNode = doc.firstChildElement("rsp");
 
     if (responseNode.attribute("stat") == "ok") {
+        QDateTime vimeoLatest = QDateTime::fromMSecsSinceEpoch(QSettings("cuteTube", "cuteTube").value("Events/vimeoLatest", 0).toLongLong());
         QDomNodeList entries = doc.elementsByTagName("video");
 
         for (int i = 0; i < entries.size(); i++) {
             QDomElement entry = entries.at(i).toElement();
             QDateTime date = QDateTime::fromString(entry.firstChildElement("upload_date").text(), Qt::ISODate);
 
-            if (date > m_vimeoLatest) {
+            if (date > vimeoLatest) {
                 Event event;
                 event.setId(entry.attribute("id"));
                 event.setSourceName("cutetube");
@@ -350,17 +341,17 @@ void Events::processVimeoEvents() {
 
 #ifdef MEEGO_EDITION_HARMATTAN
                 event.setTimestamp(date.toLocalTime().toString("yyyy-MM-dd hh:mm:ss"));
-                event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube displayVideo %1").arg(base64SerializedVariant(QVariant("http://vimeo/" + event.id()))));
+                event.setAction(QString("com.maemo.cuteTube / com.maemo.cuteTube display %1").arg(this->base64SerializedVariant(QVariant("http://vimeo.com/" + event.id()))));
 #else
                 event.setTimestamp(QString::number(date.toMSecsSinceEpoch()));
-                event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube displayVideo http://vimeo/" + event.id());
+                event.setAction("dbus:session com.maemo.cuteTube / com.maemo.cuteTube display http://vimeo.com/" + event.id());
 #endif
                 m_events.append(event);
             }
         }
 
         if (!entries.isEmpty()) {
-            m_vimeoLatest = QDateTime::fromString(entries.at(0).firstChildElement("upload_date").text(), Qt::ISODate);
+            QSettings("cuteTube", "cuteTube").setValue("Events/vimeoLatest", QDateTime::currentMSecsSinceEpoch());
         }
 #ifdef MEEGO_EDITION_HARMATTAN
         this->downloadThumbnails();
@@ -377,8 +368,6 @@ void Events::processVimeoEvents() {
 
 void Events::addEvents() {
 #ifdef MEEGO_EDITION_HARMATTAN
-    QDBusInterface iface("com.nokia.home.EventFeed", "/eventFeed", "com.nokia.home.EventFeed");
-
     while (!m_events.isEmpty()) {
         QVariantList args;
         QVariantMap itemArgs;
@@ -395,13 +384,17 @@ void Events::addEvents() {
         itemArgs.insert("action", event.action());
         args.append(itemArgs);
 
-        qDebug() << "Adding event " + event.title();
+        QDBusMessage message = QDBusMessage::createMethodCall(
+                    "com.nokia.home.EventFeed",
+                    "/eventfeed",
+                    "com.nokia.home.EventFeed",
+                    "addItem");
+        message.setArguments(args);
 
-        iface.callWithArgumentList(QDBus::NoBlock, "addItem", args);
+        QDBusConnection bus = QDBusConnection::sessionBus();
+        bus.callWithCallback(message, this, SLOT(dbusRequestCompleted(QDBusMessage)), SLOT(dbusError(QDBusError,QDBusMessage)));
     }
 #else
-    QDBusInterface iface("com.maemo.eventFeed", "/", "com.maemo.eventFeed");
-
     while (!m_events.isEmpty()) {
         QVariantList args;
         Event event = m_events.takeFirst();
@@ -410,17 +403,31 @@ void Events::addEvents() {
         args.append(event.iconPath());
         args.append(event.title());
         args.append(event.body());
-        args.append(QStringList() << event.imagePathOne());
+        args.append(QStringList(event.imagePathOne()));
         args.append(event.footer());
         args.append(event.timestamp());
         args.append(event.action());
 
-        iface.callWithArgumentList(QDBus::NoBlock, "addEvent", args);
+        QDBusInterface("com.maemo.eventFeed", "/", "com.maemo.eventFeed").callWithArgumentList(QDBus::NoBlock, "addEvent", args);
     }
 #endif
     this->syncSuccess();
 }
 
+#ifdef MEEGO_EDITION_HARMATTAN
+void Events::dbusRequestCompleted(const QDBusMessage &message) {
+    if (message.arguments().first().toInt() < 0) {
+        this->syncFailed();
+    }
+}
+
+void Events::dbusError(const QDBusError &error, const QDBusMessage &message) {
+    Q_UNUSED(error)
+    Q_UNUSED(message)
+
+    this->syncFailed();
+}
+
 QString Events::base64SerializedVariant(const QVariant &value) const
 {
     QByteArray ba;
@@ -430,22 +437,20 @@ QString Events::base64SerializedVariant(const QVariant &value) const
 }
 
 void Events::downloadThumbnails() {
-    qDebug() << m_events.size();
-    qDebug() << "Downloading thumbnail at index " + QString::number(m_index);
     if ((m_index >= 0) && (m_index < m_events.size())) {
-        Event event = m_events.at(m_index);
-        QFileInfo info(QString("/home/user/cuteTube/.feedthumbnails/%1.jpg").arg(event.id()));
+        QFileInfo info(QString("/home/user/cuteTube/.feedthumbnails/%1.jpg").arg(m_events[m_index].id()));
 
         if (info.exists()) {
-            event.setImagePathOne(info.absoluteFilePath());
+            m_events[m_index].setImagePathOne(info.absoluteFilePath());
             m_index++;
             this->downloadThumbnails();
         }
-
-        QUrl url(m_events.at(m_index).imagePathOne());
-        QNetworkRequest request(url);
-        QNetworkReply *reply = m_nam->get(request);
-        this->connect(reply, SIGNAL(finished()), this, SLOT(onThumbnailDownloaded()));
+        else {
+            QUrl url(m_events[m_index].imagePathOne());
+            QNetworkRequest request(url);
+            QNetworkReply *reply = m_nam->get(request);
+            this->connect(reply, SIGNAL(finished()), this, SLOT(onThumbnailDownloaded()));
+        }
     }
     else {
         this->addEvents();
@@ -453,7 +458,7 @@ void Events::downloadThumbnails() {
 }
 
 void Events::onThumbnailDownloaded() {
-    QNetworkReply *reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply *reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if ((!reply) || (m_index < 0) || (m_index >= m_events.size())) {
         this->syncFailed();
@@ -462,16 +467,24 @@ void Events::onThumbnailDownloaded() {
 
     QDir dir("/home/user/cuteTube/.feedthumbnails/");
     dir.mkpath(dir.path());
-    Event event = m_events.at(m_index);
-    QFile file(QString("/home/user/cuteTube/.feedthumbnails/%1.jpg").arg(event.id()));
+    QFile file(QString("/home/user/cuteTube/.feedthumbnails/%1.jpg").arg(m_events[m_index].id()));
 
     if (file.open(QIODevice::WriteOnly)) {
         file.write(reply->readAll());
         file.close();
-        event.setImagePathOne(file.fileName());
+        m_events[m_index].setImagePathOne(file.fileName());
     }
 
     reply->deleteLater();
-
+    m_index++;
     this->downloadThumbnails();
 }
+
+void Events::clearThumbnails() {
+    QDir dir("/home/user/cuteTube/.feedthumbnails/");
+
+    foreach (QString thumbnail, dir.entryList(QStringList("*.jpg"))) {
+        dir.remove(thumbnail);
+    }
+}
+#endif
